<?php

/**
 * DbFunctionsLite is a class which provides static methods in order to retrieve, insert, update and delete data
 * from tables relevant to PowerWidgets plugin
 *
 * @author Jonathan
 */
if(!class_exists('DbFunctionsLite')){
    class DbFunctionsLite {

        /*
         * returns a result object in which all pages ids and titles that are published
         */
        static function get_published_pages(){
                global $wpdb;

                return $wpdb->get_results("SELECT ID, post_title 
                    FROM " . $wpdb->prefix ."posts 
                    WHERE post_status='publish' and post_type='page'
                    ORDER BY post_title ASC", OBJECT );
        }

        static function get_published_posts() {
                global $wpdb;

                return $wpdb->get_results("SELECT ID, post_title 
                    FROM " . $wpdb->prefix ."posts 
                    WHERE post_status='publish' and post_type='post'
                    ORDER BY post_title ASC", OBJECT );
        }

        static function get_published_pages_of_title($post_title){
                global $wpdb;

                return $wpdb->get_results("SELECT ID, post_title 
                    FROM " . $wpdb->prefix ."posts 
                    WHERE post_status='publish' and post_type='page' and post_title like '%$post_title%'
                    ORDER BY post_title ASC", OBJECT );
        }

        static function get_published_specific_page($page_id){
                global $wpdb;

                return $wpdb->get_results("SELECT ID, post_title 
                    FROM " . $wpdb->prefix ."posts 
                    WHERE post_status='publish' and ID= $page_id
                    ORDER BY post_title ASC", OBJECT );
        }

        static function get_published_specific_post($post_id){
            return DbFunctionsLite::get_published_specific_page($post_id);
        }

        static function get_published_posts_of_title($post_title){
                global $wpdb;

                return $wpdb->get_results("SELECT ID, post_title 
                    FROM " . $wpdb->prefix ."posts 
                    WHERE post_status='publish' and post_type='post' and post_title like '%$post_title%'
                    ORDER BY post_title ASC", OBJECT );
        }

        static function get_defwas($defwaTableType = 'default_wa'){
                global $wpdb;

                $query = "SELECT * FROM " . $wpdb->prefix . $defwaTableType . " ORDER BY defwa_id";

                return $wpdb->get_results($query, ARRAY_A);
        }

        static function get_custwas(){
                global $wpdb;
                // check if custom_wa table exists
                if (DbFunctionsLite::checkIfTableExists($wpdb->prefix . "custom_wa")) {
                    $query = "SELECT * FROM " . $wpdb->prefix . "custom_wa ORDER BY custwa_name";

                    return $wpdb->get_results($query, ARRAY_A);
                }
                return NULL;
        }

        static function get_theme_custwas($curr_theme){
                global $wpdb;

                $query = "SELECT * FROM " . $wpdb->prefix . "custom_wa WHERE custwa_theme = '" .$curr_theme. "' ORDER BY custwa_name";

                return $wpdb->get_results($query, ARRAY_A);
        }

        /* TODO delete when moving to new design */
        static function get_theme_repwas($curr_theme){
                global $wpdb;

                $query =         "
                select posts.ID, posts.post_title, def.defwa_id, def.defwa_name, cust.custwa_name
                from " . $wpdb->prefix . "replacement_wa as rep, " . $wpdb->prefix . "posts as posts, " . $wpdb->prefix . "default_wa as def, ". $wpdb->prefix . "custom_wa as cust
                where repwa_theme = '" . $curr_theme .  "' 
                and rep.repwa_element_id = posts.ID
                and rep.defwa_id = def.defwa_id
                and rep.custwa_name= cust.custwa_name
                order by posts.post_title
                ";

                return $wpdb->get_results($query, OBJECT);
        }

        static function getThemeRepwas($theme){
                global $wpdb;

                $query = "
                    select *
                    from " . $wpdb->prefix . "replacement_wa as rep 
                    where repwa_theme = '" . $theme .  "' 
                ";

                return $wpdb->get_results($query, OBJECT);
        }

        static function get_theme_reptype_repwas($curr_theme, $rep_type){
                global $wpdb;

                $page_or_post_query =         "
                select rep.repwa_element_id, posts.post_title, def.defwa_id, def.defwa_name, cust.custwa_name
                from " . $wpdb->prefix . "replacement_wa as rep, " . $wpdb->prefix . "posts as posts, " . $wpdb->prefix . "default_wa as def, ". $wpdb->prefix . "custom_wa as cust
                where repwa_theme = '" . $curr_theme .  "'
                and rep.repwa_theme = def.defwa_theme
                and rep.repwa_type = '" . $rep_type . "' 
                and rep.repwa_element_id = posts.ID
                and rep.defwa_id = def.defwa_id
                and rep.custwa_name= cust.custwa_name
                order by posts.post_title
                ";

                $page_template_query = 
                "
                select rep.repwa_element_id, def.defwa_id, def.defwa_name, cust.custwa_name
                from " . $wpdb->prefix . "replacement_wa as rep, " . $wpdb->prefix . "default_wa as def, ". $wpdb->prefix . "custom_wa as cust
                where repwa_theme = '" . $curr_theme .  "' 
                and rep.repwa_theme = def.defwa_theme
                and rep.repwa_type = '" . $rep_type . "' 
                and rep.defwa_id = def.defwa_id
                and rep.custwa_name= cust.custwa_name
                order by rep.repwa_element_id
                ";

                $all_page_templates_query = 
                "
                select rep.repwa_element_id, def.defwa_id, def.defwa_name, cust.custwa_name
                from " . $wpdb->prefix . "replacement_wa as rep, " . $wpdb->prefix . "default_wa as def, ". $wpdb->prefix . "custom_wa as cust
                where repwa_theme = '" . $curr_theme .  "' 
                and rep.repwa_theme = def.defwa_theme
                and (rep.repwa_type = 'blog_page' 
                    or rep.repwa_type = '404_page' 
                    or rep.repwa_type = 'search_results_page' 
                    or rep.repwa_type = 'date_archive' 
                ) 
                and rep.defwa_id = def.defwa_id
                and rep.custwa_name= cust.custwa_name
                order by rep.repwa_element_id
                ";

                if ($rep_type === 'post' || $rep_type === 'page'){
                    return $wpdb->get_results($page_or_post_query, OBJECT);
                }

                else if ($rep_type === 'page_template'){
                    return $wpdb->get_results($all_page_templates_query, OBJECT);
                }

                else{
                    return $wpdb->get_results($page_template_query, OBJECT);
                }
        }

        /* TODO - delete get_theme_post_repwas method when moving to replaceable managers */
        static function get_theme_post_repwas($curr_theme, $post_id){
                global $wpdb;

                $query = "
                    select posts.ID, posts.post_title, def.defwa_id, def.defwa_name, cust.custwa_name
                    from " . $wpdb->prefix . "replacement_wa as rep, " . $wpdb->prefix . "posts as posts, " . $wpdb->prefix . "default_wa as def, ". $wpdb->prefix . "custom_wa as cust
                    where repwa_theme = '" . $curr_theme .  "' 
                    and rep.repwa_element_id = posts.ID
                    and rep.repwa_element_id = '" .$post_id."'
                    and (rep.repwa_type = 'page' or rep.repwa_type = 'post') 
                    and rep.defwa_id = def.defwa_id
                    and rep.custwa_name= cust.custwa_name
                    order by posts.post_title
                ";

                return $wpdb->get_results($query, OBJECT);
        }

        static function getPostRepwasByTheme( $post_id, $curr_theme) {
                global $wpdb;

                $query = "
                    select *
                    from " . $wpdb->prefix . "replacement_wa as rep 
                    where repwa_theme = '" . $curr_theme .  "' 
                    and rep.repwa_element_id = '" .$post_id."'
                    and (rep.repwa_type = 'page' or rep.repwa_type = 'post')
                ";

                return $wpdb->get_results($query, OBJECT);
        }

        /* TODO delete get_theme_category_repwas when moving to pwReplaceable managers */
        static function get_theme_category_repwas($curr_theme, $category_id){
                global $wpdb;

                $query = 
                    "
                        select rep.repwa_element_id, def.defwa_id, def.defwa_name, cust.custwa_name
                        from " . $wpdb->prefix . "replacement_wa as rep, " . $wpdb->prefix . "default_wa as def, ". $wpdb->prefix . "custom_wa as cust
                        where repwa_theme = '" . $curr_theme .  "' 
                        and rep.repwa_element_id = '" .$category_id."'
                        and rep.repwa_type = 'category' 
                        and rep.defwa_id = def.defwa_id
                        and rep.custwa_name= cust.custwa_name
                        order by rep.repwa_element_id
                    ";

                return $wpdb->get_results($query, OBJECT);
        }

        static function getCategoryRepwasByTheme($curr_theme, $category_id){
                global $wpdb;

                $query = "
                    SELECT *
                    FROM " . $wpdb->prefix . "replacement_wa as rep 
                    WHERE repwa_theme = '" . $curr_theme .  "' 
                    AND rep.repwa_element_id = '" .$category_id."'
                    AND rep.repwa_type = 'category'
                ";

                return $wpdb->get_results($query, OBJECT);
        }

        /* TODO delete get_theme_tag_repwas when moving to pwReplaceable managers */
        static function get_theme_tag_repwas($curr_theme, $tag_id){
                global $wpdb;

                $query =         "
                select rep.repwa_element_id, def.defwa_id, def.defwa_name, cust.custwa_name
                from " . $wpdb->prefix . "replacement_wa as rep, " . $wpdb->prefix . "default_wa as def, ". $wpdb->prefix . "custom_wa as cust
                where repwa_theme = '" . $curr_theme .  "' 
                and rep.repwa_element_id = '" .$tag_id."'
                and rep.repwa_type = 'tag' 
                and rep.defwa_id = def.defwa_id
                and rep.custwa_name= cust.custwa_name
                order by rep.repwa_element_id
                ";

                return $wpdb->get_results($query, OBJECT);
        }

        static function getTagRepwasByTheme($curr_theme, $tagId){
                global $wpdb;

                $query = "
                    SELECT *
                    FROM " . $wpdb->prefix . "replacement_wa as rep 
                    WHERE repwa_theme = '" . $curr_theme .  "' 
                    AND rep.repwa_element_id = '" .$tagId."'
                    AND rep.repwa_type = 'tag'
                ";

                return $wpdb->get_results($query, OBJECT);
        }

        /* TODO delete get_theme_page_template_repwas when moving to pwReplaceable managers */
        static function get_theme_page_template_repwas($curr_theme, $rep_type){
                global $wpdb;

                $query =         "
                select rep.repwa_element_id, def.defwa_id, def.defwa_name, cust.custwa_name
                from " . $wpdb->prefix . "replacement_wa as rep, " . $wpdb->prefix . "default_wa as def, ". $wpdb->prefix . "custom_wa as cust
                where repwa_theme = '" . $curr_theme .  "' 
                and rep.repwa_type = '" . $rep_type .  "' 
                and rep.defwa_id = def.defwa_id
                and rep.custwa_name= cust.custwa_name
                order by rep.repwa_element_id
                ";

                return $wpdb->get_results($query, OBJECT);
        }

        static function getPageTemplateRepwasByTheme ($currTheme, $ptId, $ptType){
                global $wpdb;

                $query = "
                    SELECT *
                    FROM " . $wpdb->prefix . "replacement_wa as rep 
                    WHERE repwa_theme = '" . $currTheme .  "' 
                    AND rep.repwa_element_id = '" .$ptId."'
                    AND rep.repwa_type = '" .$ptType."'
                ";

                return $wpdb->get_results($query, OBJECT);
        }

        static function get_theme_defwas($curr_theme){
                global $wpdb;

                 return $wpdb->get_results("select defwa_id, defwa_name from " . $wpdb->prefix . "default_wa where defwa_theme = '" .$curr_theme. "'", OBJECT);
        }

        static function get_plugin_license(){
            return get_option(PowerWidgets::LICENSE_OPTION, null);
        }



        static function set_plugin_license($new_license){
            return update_option(PowerWidgets::LICENSE_OPTION, $new_license);
        }

        static function delete_custwa_repwas($curr_theme, $custwa_name){
                global $wpdb;

                return $wpdb->query(
                        $wpdb->prepare(
                        "delete  from " . $wpdb->prefix . "replacement_wa 
                            where repwa_theme = %s
                            and  custwa_name = %s",
                            $curr_theme,
                            $custwa_name
                        )
                    );
        }

        static function delete_custwa($curr_theme, $custwa_name){
                global $wpdb;

                return $wpdb->query(
                        $wpdb->prepare(
                            "DELETE FROM " . $wpdb->prefix . "custom_wa 
                             WHERE custwa_name = %s",
                             $custwa_name
                        )
                    );
        }

        static function delete_element_repwas_of_theme($curr_theme, $element_id, $rep_type){
                global $wpdb;

                return $wpdb->query(
                        $wpdb->prepare(
                        "delete  from " . $wpdb->prefix . "replacement_wa 
                            where repwa_theme = %s
                            and repwa_element_id = %d
                            and repwa_type = %s
                        ",
                            $curr_theme,
                            $element_id, 
                            $rep_type
                        )
                    );
        }

        static function delete_element_repwas($element_id, $rep_type){
                global $wpdb;

                return $wpdb->query(
                        $wpdb->prepare(
                        "delete  from " . $wpdb->prefix . "replacement_wa 
                            where repwa_element_id = %d
                            and repwa_type = %s
                        ",
                            $element_id, 
                            $rep_type
                        )
                    );
        }

        /*
         * Deletes all $curr_theme defwas
         */
        function deleteDefwas($defwasTableType){
                    global $wpdb;

                    return $wpdb->query(
                                "delete  from " . $wpdb->prefix . $defwasTableType
                        );
        }

        static function insert_new_custwa($c) {
                global $wpdb;

                $wpdb->insert( 
                    $wpdb->prefix . 'custom_wa', 
                    array( 
                        'custwa_name' => $c->get_name(), 
                        'custwa_description' => $c->get_description(), 
                        'custwa_theme' => $c->get_theme(), 
                        'custwa_class' => $c->get_class(), 
                        'custwa_before_widget' => $c->get_bewidget(), 
                        'custwa_after_widget' => $c->get_afwidget(), 
                        'custwa_before_title' => $c->get_betitle(), 
                        'custwa_after_title' => $c->get_aftitle()
                    ), 
                    array(
                            '%s', 
                            '%s',                            
                            '%s', 
                            '%s', 
                            '%s',                            
                            '%s', 
                            '%s', 
                            '%s' 
                    )
                );
        }

        static function insert_row_repwas($curr_theme, $page_id, $repwas_arr, $post_type){
                global $wpdb;

                foreach ($repwas_arr as $defwa=>$custwa){
                    $custwa = str_replace('_', ' ', $custwa);

                    $wpdb->insert( 
                            $wpdb->prefix . "replacement_wa", 
                            array( 
                                    'defwa_id' => $defwa, 
                                    'custwa_name' => $custwa, 
                                    'repwa_theme' => $curr_theme, 
                                    'repwa_element_id' => $page_id, 
                                    'repwa_type' => $post_type
                            ), 
                            array( 
                                    '%s', 
                                    '%s', 
                                    '%s', 
                                    '%d', 
                                    '%s' 
                            ) 
                    );
                }
        }


        /**
         * inserts a repwa to the db
         * @global type $wpdb
         * @param Repwa $repwa a Repwa object to be inserted to db
         */
        public static function insertRepwa ($repwa) {
                global $wpdb;

                $wpdb->insert( 
                        $wpdb->prefix . "replacement_wa", 
                        array( 
                                'defwa_id' => $repwa->get_defwa()->get_id(), 
                                'custwa_name' => $repwa->get_custwa()->get_name(), 
                                'repwa_theme' => $repwa->get_repwaTheme(), 
                                'repwa_element_id' => $repwa->get_repwaElemId(), 
                                'repwa_type' => $repwa->get_repwaElemType()
                        ), 
                        array( 
                                '%s', 
                                '%s', 
                                '%s', 
                                '%d', 
                                '%s' 
                        ) 
                );
        }   // end function insertRepwa

        /**
         * inserts a repwa to the db or updates an existing repwa with unique key
         * @global type $wpdb
         * @param Repwa $repwa a Repwa object to be inserted to db
         * @return void
         */
        public static function insertOrUpdateRepwa ($repwa) {
                global $wpdb;

                //prepare query
                $tableName = $wpdb->prefix . "replacement_wa";
                $query = 
                    "INSERT INTO `$tableName` (`defwa_id`, `custwa_name`, `repwa_theme`, `repwa_element_id`, `repwa_type`) 
                    VALUES(%s, %s, %s, %d, %s) ON DUPLICATE KEY UPDATE
                        `custwa_name` = '" . $repwa->get_custwa()->get_name() . "',  
                        `repwa_theme` = '" . $repwa->get_repwaTheme() . "'";
                ;

                // execute query
                $wpdb->query(
                        $wpdb->prepare (
                                $query,
                                $repwa->get_defwa()->get_id(),
                                $repwa->get_custwa()->get_name(),
                                $repwa->get_repwaTheme(), 
                                $repwa->get_repwaElemId(),
                                $repwa->get_repwaElemType()
                        )
                );
        }

        /*
         * update defwas in db
         */
        static function update_defwas_in_db($defwas, $old = FALSE){
                global $wpdb;

                if ($old ) { $defwasTableType = 'old_default_wa' ;}
                else {$defwasTableType = 'default_wa'; }

                //remove all defwas
                DbFunctionsLite::deleteDefwas($defwasTableType);

                // fill defwa table with current theme data
                foreach ($defwas->getList() as $defwa) {
                    $defwa_id = $defwa->get_id();
                    $defwa_name = $defwa->get_name();
                    $defwa_theme = $defwa->get_theme();
                    $defwa_description = $defwa->get_description();
                    $defwa_class= $defwa->get_class();
                    $defwa_before_widget = $defwa->get_bewidget();
                    $defwa_after_widget = $defwa->get_afwidget();
                    $defwa_before_title = $defwa->get_betitle();
                    $defwa_after_title = $defwa->get_aftitle();

                    $query = 
                                "INSERT IGNORE INTO `" .$wpdb->prefix . $defwasTableType . "` (`defwa_id`, `defwa_theme`, `defwa_name`, `defwa_description`, `defwa_class`, `defwa_before_widget`, `defwa_after_widget`, `defwa_before_title`, `defwa_after_title`) 
                                    VALUES(%s, %s, %s, %s, %s, %s, %s, %s, %s)";
                    $wpdb->query(
                            $wpdb->prepare( 
                                    $query,
                                    $defwa_id,
                                    $defwa_theme,
                                    $defwa_name,
                                    $defwa_description,
                                    $defwa_class,
                                    $defwa_before_widget,
                                    $defwa_after_widget,
                                    $defwa_before_title,
                                    $defwa_after_title
                            ) 
                    );
                }

        }   // end function update_defwas_in_db

        /*------------------------------------------------------------------------------------------------------------------------*/
        /**
         * updates specific custwa record (record of custwa name $oldCustwaName) with info found at $newCustwa
         * @param string $oldCustwaName old custwa name
         * @param Custwa $newCustwa new custwa object
         */
        static function updateCustwaInCustwasTable($oldCustwaName, $newCustwa) {
            global $wpdb;

                $wpdb->update( 
                        $wpdb->prefix . 'custom_wa', 
                        array(
                                'custwa_name' => $newCustwa->get_name(),
                                'custwa_description' => $newCustwa->get_description(),
                                'custwa_theme' => $newCustwa->get_theme(),
                                'custwa_class' => $newCustwa->get_class(),
                                'custwa_before_widget' => $newCustwa->get_bewidget(),
                                'custwa_after_widget' => $newCustwa->get_afwidget(),
                                'custwa_before_title' => $newCustwa->get_betitle(),
                                'custwa_after_title' => $newCustwa->get_aftitle()
                        ),
                        array( 'custwa_name' => $oldCustwaName ), 
                        array( 
                                '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s'
                        ), 
                        array( '%s' )
                );
        }

        /*------------------------------------------------------------------------------------------------------------------------*/
        /**
         * updates specific custwa record (record of custwa name $oldCustwaName) with info found at $newCustwa
         * @param string $oldCustwaName old custwa name
         * @param Custwa $newCustwa new custwa object
         * @return void
         */
        static function updateCustwaInRepwasTable($oldCustwaName, $newCustwa) {
            global $wpdb;

                $wpdb->update ( 
                        $wpdb->prefix . 'replacement_wa', 
                        array(
                                'custwa_name' => $newCustwa->get_name()
                        ),
                        array( 'custwa_name' => $oldCustwaName ), 
                        array( '%s' ), 
                        array( '%s' )
                );
        }

    /*------------------------------------------------------------------------------------------------------------------------*/
         /**
          * checks if table exists in the database
          * @global type $wpdb
          * @param string $tableName
          *     the table name to look for
          * @return boolean
          *     true - if table exists and false otherwise
          */
         public static function checkIfTableExists($tableName) {
            global $wpdb;
            if($wpdb->get_var("SHOW TABLES LIKE '$tableName'") != $tableName) {
                return FALSE;
            }
            return TRUE;
         }

        /*
         * TODO update valid_build only once
         */
        static function create_curr_blog_pw_tables($curr_blog_ver, $desired_blog_ver){
                global $wpdb;
                global $wp_registered_sidebars;

                $valid_build = false;
                while ($curr_blog_ver < $desired_blog_ver){
                    switch ($curr_blog_ver){
                        case 0:

                            // insert default_wa table
                            $table_name = $wpdb->prefix . "default_wa";
                            $insert_table_query = "CREATE TABLE IF NOT EXISTS `" . $table_name . "` (
                          `defwa_id` varchar(100) NOT NULL,
                          `defwa_theme` varchar(50) NOT NULL,
                          `defwa_name` varchar(100) NOT NULL,
                          `defwa_description` varchar(1000) NOT NULL,
                          `defwa_class` varchar(1000) NOT NULL,
                          `defwa_before_widget` varchar(1000) NOT NULL,
                          `defwa_after_widget` varchar(1000) NOT NULL,
                          `defwa_before_title` varchar(1000) NOT NULL,
                          `defwa_after_title` varchar(1000) NOT NULL,
                          PRIMARY KEY (`defwa_id`,`defwa_theme`)
                        ) ENGINE=InnoDB DEFAULT CHARSET=utf8";

                            $wpdb->query(  $insert_table_query );

                            // insert custom_wa table
                            $table_name = $wpdb->prefix . "custom_wa";
                            $insert_table_query = "CREATE TABLE IF NOT EXISTS `" . $table_name . "` (
                          `custwa_name` varchar(100) NOT NULL,
                          `custwa_description` varchar(1000) NOT NULL,
                          `custwa_theme` varchar(50) NOT NULL,
                          `custwa_class` varchar(1000) NOT NULL,
                          `custwa_before_widget` varchar(1000) NOT NULL,
                          `custwa_after_widget` varchar(1000) NOT NULL,
                          `custwa_before_title` varchar(1000) NOT NULL,
                          `custwa_after_title` varchar(1000) NOT NULL,
                          PRIMARY KEY (`custwa_name`)
                        ) ENGINE=InnoDB DEFAULT CHARSET=utf8";

                        $wpdb->query(  $insert_table_query );

                            // insert replacement_wa table
                            $table_name = $wpdb->prefix . "replacement_wa";
                            $insert_table_query = "CREATE TABLE IF NOT EXISTS `" . $table_name . "` (
                            `repwa_id` bigint(20) NOT NULL AUTO_INCREMENT,
                            `defwa_id` varchar(30) NOT NULL,
                            `custwa_name` varchar(30) NOT NULL,
                            `repwa_theme` varchar(50) NOT NULL,
                            `repwa_element_id` bigint(20) NOT NULL,
                            `repwa_type` varchar(30) NOT NULL, 
                            PRIMARY KEY (`repwa_id`)
                          ) ENGINE=InnoDB  DEFAULT CHARSET=utf8";

                            $wpdb->query(  $insert_table_query );

                            //TODO in overriding_prot_wa change prot_id type to be bigint 20 instead of  varchar 200 - change in case 0 also
                            // insert overriding_prot_wa table
                            $table_name = $wpdb->prefix . "overriding_prot_wa";
                            $insert_table_query = "CREATE TABLE IF NOT EXISTS `" . $table_name . "` (
                          `element_id` bigint(20) unsigned NOT NULL,
                          `element_type` varchar(32) NOT NULL,
                          `prot_id` bigint(20) NOT NULL,
                          PRIMARY KEY (`element_id`,`element_type`)
                        ) ENGINE=InnoDB DEFAULT CHARSET=utf8";

                            $wpdb->query(  $insert_table_query );

                            // insert prototypes_wa table
                            $table_name = $wpdb->prefix . "prototypes_wa";
                            $insert_table_query = 
                                "CREATE TABLE IF NOT EXISTS `" . $table_name . "` (
                                `prot_id` bigint(20) NOT NULL AUTO_INCREMENT,
                                `prot_title` varchar(32) NOT NULL,
                                PRIMARY KEY (`prot_id`)
                              ) ENGINE=InnoDB DEFAULT CHARSET=utf8";

                            $wpdb->query(  $insert_table_query );

                            // fill defwa table with current theme data
                            $defwas = DefwaList::standard_registered_sidebars_to_wa_list($wp_registered_sidebars, wp_get_theme());
                            DbFunctionsLite::update_defwas_in_db($defwas, FALSE);

                            // add pw control table direction option according to the blog language

                            //check if there is no pw direction option.
                            if (!get_option('pw_dir', NULL)){
                                //get language
                                $language = get_bloginfo( 'language' );
                                // initialize direction to left to right
                                $dir = 'ltr';
                                // if the language is rtl
                                if ($language === 'he_IL'){
                                    $dir = 'rtl';
                                }
                                add_option( 'pw_dir', $dir, '', 'no' );
                            }

                            // actually now we are in the same final result of case 2
                            $curr_blog_ver = $curr_blog_ver + 1;

                            // if one got here - tables were built o.k.
                            $valid_build = true;
                            break;  // break case 0

                            case 1:

                                // insert prototypes_wa table
                                $table_name = $wpdb->prefix . "prototypes_wa";
                                $insert_table_query = 
                                    "CREATE TABLE IF NOT EXISTS `" . $table_name . "` (
                                    `prot_id` bigint(20) NOT NULL AUTO_INCREMENT,
                                    `prot_title` varchar(32) NOT NULL,
                                    PRIMARY KEY (`prot_id`)
                                  ) ENGINE=InnoDB DEFAULT CHARSET=utf8";
                                $wpdb->query(  $insert_table_query );

                                // rename table wp_overriding_tax_wa  to wp_overriding_prot_wa
                                $table_name_from = $wpdb->prefix . "overriding_tax_wa";
                                $table_name_to = $wpdb->prefix . "overriding_prot_wa";
                                $rename_table_query = "RENAME TABLE " . $table_name_from . " TO $table_name_to";
                                $wpdb->query (  $rename_table_query );

                                // rename col post_id to element_id
                                $table_name = $wpdb->prefix . "overriding_prot_wa";
                                $old_column_name = "post_id";
                                $new_col_name = "element_id";
                                $rename_table_query = "ALTER TABLE `$table_name`  CHANGE `$old_column_name` `$new_col_name` BIGINT( 20 ) UNSIGNED NOT NULL";
                                $wpdb->query (  $rename_table_query );

                                // rename col tax_type to element_type
                                $table_name = $wpdb->prefix . "overriding_prot_wa";
                                $old_column_name = "tax_type";
                                $new_col_name = "element_type";
                                $rename_table_query = "ALTER TABLE `$table_name`  CHANGE `$old_column_name` `$new_col_name` VARCHAR( 32 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL";
                                $wpdb->query (  $rename_table_query );

                                // rename col tax_value to prot_id
                                $table_name = $wpdb->prefix . "overriding_prot_wa";
                                $old_column_name = "tax_value";
                                $new_col_name = "prot_id";
                                $rename_table_query = "ALTER TABLE `$table_name`  CHANGE `$old_column_name` `$new_col_name` BIGINT( 20 ) NOT NULL";
                                $wpdb->query (  $rename_table_query );

                                // if one got here - tables were built o.k.
                                $valid_build = true;

                                break;  // break case 1
                            case 2:
                                $websitePrefix = $wpdb->prefix;
                                $qry_add_prot_theme = "ALTER TABLE  `" . $websitePrefix . "prototypes_wa` ADD  `prot_theme` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL AFTER  `prot_title`";
                                $wpdb->query (  $qry_add_prot_theme );

                                // if one got here - tables were built o.k.
                                $valid_build = true;
                                break;
                            case 3:
                                    // insert old theme defwas table
                                    $table_name = $wpdb->prefix . "old_default_wa";
                                    $qry_insert_old_theme_defwas = "CREATE TABLE IF NOT EXISTS `" . $table_name . "` (
                                        `defwa_id` varchar(100) CHARACTER SET utf8 NOT NULL,
                                        `defwa_theme` varchar(50) CHARACTER SET utf8 NOT NULL,
                                        `defwa_name` varchar(100) CHARACTER SET utf8 NOT NULL,
                                        `defwa_description` varchar(1000) CHARACTER SET utf8 NOT NULL,
                                        `defwa_class` varchar(1000) CHARACTER SET utf8 NOT NULL,
                                        `defwa_before_widget` varchar(1000) CHARACTER SET utf8 NOT NULL,
                                        `defwa_after_widget` varchar(1000) CHARACTER SET utf8 NOT NULL,
                                        `defwa_before_title` varchar(1000) CHARACTER SET utf8 NOT NULL,
                                        `defwa_after_title` varchar(1000) CHARACTER SET utf8 NOT NULL,
                                        PRIMARY KEY (`defwa_id`,`defwa_theme`)
                                    ) ENGINE=InnoDB DEFAULT CHARSET=utf8";
                                    $wpdb->query (  $qry_insert_old_theme_defwas );

                                    // insert old theme option
                                    add_option('pw_old_theme', NULL);

                                    // if one got here - tables were built o.k.
                                    $valid_build = true;
                                    break;

                            case 4:
                                    $table_name = $wpdb->prefix . "replacement_wa";
                                    $addReplacementUniqueKeyQry = "
                                            ALTER TABLE  `$table_name` ADD UNIQUE (
                                            `defwa_id` ,
                                            `repwa_theme` ,
                                            `repwa_element_id` ,
                                            `repwa_type`
                                            );
                                        ";

                                    $wpdb->query (  $addReplacementUniqueKeyQry );

                                    // if one got here - tables were built o.k.
                                    $valid_build = true;
                                    break;

                            case 5:
                                    $table_name = $wpdb->prefix . "overriding_prot_wa";
                                    $changePrimaryKeyInOverrideProtTable = "
                                            ALTER TABLE `$table_name`
                                              DROP PRIMARY KEY,
                                               ADD PRIMARY KEY(
                                                 `element_id`,
                                                 `element_type`,
                                                 `prot_id`);
                                        ";

                                    $wpdb->query (  $changePrimaryKeyInOverrideProtTable );

                                    // if one got here - tables were built o.k.
                                    $valid_build = true;
                                    break;
                            case 6:
                                    delete_option('power_widgets_key');

                                    // if one got here - tables were built o.k.
                                    $valid_build = true;
                                    break;

                            case 7:
                                $qryCheckIfProtThemeExists = "SHOW columns from `" . $websitePrefix . "prototypes_wa` where field='prot_theme'";
                                try {
                                    $rows = $wpdb->query (  $qryCheckIfProtThemeExists );
                                    if (empty( $rows )) {
                                        $websitePrefix = $wpdb->prefix;
                                        $qry_add_prot_theme = "ALTER TABLE  `" . $websitePrefix . "prototypes_wa` ADD  `prot_theme` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL AFTER  `prot_title`";
                                        $wpdb->query (  $qry_add_prot_theme );
                                    }
                                }
                                catch (Exception $e) {
                                    // do nothing
                                }

                                // if one got here - tables were built o.k.
                                $valid_build = true;
                                break;
                                
                                case 8:
                                	$qryAlterCustwaAndDefwaIdsVarcharLength =
                                	"
										ALTER TABLE `" . $wpdb->prefix . "replacement_wa` CHANGE `defwa_id` `defwa_id` VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL, CHANGE `custwa_name` `custwa_name` VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL;
                                	";
                                	try {
                                		$wpdb->query (  $qryAlterCustwaAndDefwaIdsVarcharLength );
                                	}
                                	catch (Exception $e) {
                                		// do nothing
                                	}
                                
                                	// if one got here - tables were built o.k.
                                	$valid_build = true;
                                	break;
                        }

                        $curr_blog_ver = $curr_blog_ver + 1;
                    }
                    return $valid_build;
                return false;
        }   //end create_curr_blog_pw_tables


        /* remains */
         public static function truncateTable($tableName) {
             global $wpdb;

             $qry = "truncate table `$tableName`";
             $wpdb->query (  $qry );
         }

    } //end DbFunctionsLite class
}   // end if class exists

?>