<?php

/**
 * PwList implements a power widgets list 
 *
 * @author Jonathan
 */
if(!class_exists('PwList')){
    class PwList {
        private $list;
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
    /**
     * PwList constructor - builds a new PwList object contain empty list
     * @Return PwList new empty PwList.
     */
        public function __construct() {
            $this->list = array();
        }
        
    /**
     * PwList constructor - builds a new PwList object contain empty list
     * @Return PwList new empty PwList.
     */
        public static function __constructWithPwList($anotherList) {
            $this->list = $anotherList->getList();
        }
        
    /**
     * PwList constructor - builds a new PwList object contain empty list
     * @Return PwList new empty PwList.
     */
        public static function __constructWithElement($elem) {
            $this->list = new PwList();
            $this->set(0, $elem);
        }
        
//-------------------------------------------------getters-------------------------------------------------
    
        function getList(){
            return $this->list;
        }
    
//-------------------------------------------------end getters-------------------------------------------------    
//-------------------------------------------------setters-------------------------------------------------  
    
        function setList($value) {
            $this->list = $value;
        }
//-------------------------------------------------end setters-------------------------------------
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * Returns the element at the specified position in this list.
         * @param int $index
         *      index of element to return.
         * @return Object
         *      the element at the specified position in this list.
         */
        public function get($index) {
            return $this->list[$index];
        }
        
        /**
         * Returns the element at the specified position in this list.
         * @param int $index
         *      index of element to return.
         * @return Object
         *      the element at the specified position in this list.
         */
        public function getByValue($elem) {
            return $this->get($this->indexOf($elem));
        }
        
        /**
         * Replaces the element at the specified position in this list with the specified element.
         * @param int $index
         *      index of element to replace.
         * @param type $elem
         *      the replaceing element at the specified position.
         */
        public function set($index, $elem) {
            $this->list[$index] = $elem;
        }
        
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * adds an element to the elements list class member
         * @param WidgetAreaPw $element the new element to add
         * @return void
         */
        public function add ($element) {
            $list = $this->getList();
            $this->list[] = $element;
        }   // end function add_wa_to_list
        
        /**
         * Removes the element at the specified position in this list.
         * Shifts any subsequent elements to the left (subtracts one from their indices).
         * @param Object $element
         * @return Object
         *      the element that was removed from the list.
         */
        public function removeByIndex ($index) {
            unset ( $this->list[$index] );
        }
        
        public function remove ($element) {
            $index = $this->indexOf($element);
            removeByIndex ($index);
        }
        
        /**
         * Searches for the first occurence of the given argument, testing for equality using the equals method.
         * @param Object $elem
         * @return int
         *      the index of the last occurrence of the specified object in this list; returns -1 if the object is not found.
         */
        public function indexOf ($elem) {
            foreach ($this->getList() as $index=>$thisElem) {
                if ( $thisElem == $elem ) {
                    return $index;
                }
            }
            return -1;
        }
          
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * subtracts a given list $subtracted_list from this list
         * @param PwList $subtracted_list  
         *      the list which is subtracted from this list:
         * @return PwList 
         *      the subtracted list in case of success. 
         *      in case subtracted list is empty or NULL this list will be returned.
         */
        public function subtract(PwList $subList) {
            // if subtracted list is null
            if ( is_null($subList) ) {  return $this; }

            $subListArr = $subList->getList();
            // if subtracted list arr is null or empty
            if ( is_null($subListArr ) || count ( $subListArr ) === 0) {
                return $this;
            }
            
            // end of input validation
            
            // subtract elements from $subList
            foreach ($subListArr as $element) {
                $this->remove ($element);
            }
        }   // end function subtract
        
        
    }   // end class PwList
} // end if class PwList not exist

?>
