<?php

/**
 * RepwasAjaxHandler handles ajax requests regarding repwas management
 *
 * @author Jonathan
 */

require_once(pw_repwas . '/gui/LogicRepwasTable.php');
require_once(pw_repwas . '/gui/CustwasDropdown.php');

class RepwasAjaxHandler {
        /**
         * Constructor of RepwasAjaxHandler 
         */
        public function __construct() {
            
            /* actions */
            // ajax hook for handling http reqs of jeasyui get table
            add_action('wp_ajax_get_repwas_table',  array($this, 'handleGetRepwasTable'), 10);
            
            // ajax hook for handling http reqs of jeasyui update repwas table row
            add_action('wp_ajax_update_repwas_table_row',  array($this, 'handleUpdateRepwasTableRow'), 10);
            
            // ajax hook for handling http reqs of jeasyui save repwas table row
            add_action('wp_ajax_save_repwas_table_row',  array($this, 'handleSaveRepwasTableRow'), 10);
            
            // ajax hook for handling http reqs of jeasyui delete repwas table row
            add_action('wp_ajax_delete_repwas_table_row',  array($this, 'handleDeleteRepwasTableRow'), 10);
            
            // ajax hook for handling http reqs of jeasyui get custwas dropdown
            add_action('wp_ajax_get_custwas_dropdown',  array($this, 'handleGetCustwasDropdown'), 10);
        } // end WaManager Constructor
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
    /**
     * handles a get table ajax request by generating the table gui component and echoing it.
     * @param none
     * @return none
     */
    public function handleGetRepwasTable() {
        
        // get parameters from client
        $rep_type = $_GET['rep_type'];
        $edited_post = intval (isset($_GET['edited_post']) ? mysql_real_escape_string($_GET['edited_post']) : NULL);
        $element_name = isset($_POST['replaceable_element']) ? mysql_real_escape_string($_POST['replaceable_element']) : NULL;
        $page = isset($_POST['page']) ? intval($_POST['page']) : 1;  
        $rows = isset($_POST['rows']) ? intval($_POST['rows']) : 10;
        $offset = ($page-1)*$rows;

        // create table
        $this->createTable( $rep_type, $edited_post, $element_name, $offset, $rows);
        
        // echo table
        echo $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasGuiManager()->get_currTable();
        
        // die because it's ajax req
        die();
    }    
    
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
    /**
     * handles update row ajax request by retrieving the row repwas data and updating the db elements
     * to fit with the repwas data retrieved. function also echoes the parameters received in order to show 
     * them on the table after update operation took place.
     * @param none
     * @return none
     */
    public function handleUpdateRepwasTableRow() {
        
        // get parameters from client
        $curr_theme = $_GET['current_theme'];   //current theme
        $element_id = intval($_REQUEST['replaceable_element_id']); //element id
        $rep_type = $_GET['rep_type'];  // repwa type

        // update table row
        $this->updateRepwasTableRow( $curr_theme, $element_id, $rep_type);

        //echo the json to the server
        echo json_encode($_REQUEST);
        
        // die because it's ajax req
        die();
    }
    
    
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
    /**
     * function handles get custwas request (when user dbl clicks a defwa column and expects to see a custwa list)
     */
    public function handleGetCustwasDropdown() {
        // create table
        $this->createCustwasDropDown();
        
        // echo table
        echo $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasGuiManager()->get_custwasDropdown();
        
        // die because it's ajax req
        die();
    }   // end function handleGetCustwasDropdown
    
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
    /**
     * function creates a repwas table of a specific element type
     * @param string $reptype
     *      the specific element type according to the control table is built
     * @param int $edited_post
     *      the edited post in case we are in edit page/post mode
     * @param string $element_name
     *      the element type (page, post, category, tag, page_template)
     * @return none
     */
    private function createTable( $rep_type, $edited_post, $element_name, $offset, $rows) {
            $logicRepwastable = new LogicRepwasTable ($rep_type, $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_defwas(), $element_name, $edited_post );
            $repwasTable = new RepwasTable($logicRepwastable, $offset, $rows);
            $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasGuiManager()->set_currTable($repwasTable);
    }   // end createAndEchoTable
    
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
    /**
     * function creates a dropdown custwas list
     * @param string $reptype
     *      the specific element type according to the control table is built
     * @return none
     */
    private function createCustwasDropDown () {
            $custwasList = $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_custwas();
            $custwasDd = new CustwasDropdown($custwasList);
            $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasGuiManager()->set_custwasDropdown($custwasDd);
    }
    
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
    private function updateRepwasTableRow( $curr_theme, $element_id, $rep_type) {
        // if repwa type is "page_template" then change it to the specific page template relevant
        if ($rep_type === "page_template") {
            $rep_type = $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasPageTemplatesManager()->get_pwPageTemplates()->getById($element_id)->getType();
        }

        // get current row's empty repwas and current row's non empty repwas
        $non_empty_repwas = array();
        $empty_repwas = array();
        $this->get_empty_and_nonempty_repwas($empty_repwas, $non_empty_repwas);

        // delete page's repwas
        DbFunctionsLite::delete_element_repwas_of_theme($curr_theme, $element_id, $rep_type);
        // reinsert page's non empty repwas
        DbFunctionsLite::insert_row_repwas($curr_theme, $element_id, $non_empty_repwas, $rep_type);
    }   // end function updateRepwasTableRow

    /**
     * sets 
     * param $curr_theme - the current theme
     * param $empty_repwas - the array in which the empty repwas enter
     * param $non_empty_repwas - the array in which the non empty repwas enter
     * goes over all row repwas and divides the repwas into empty and non empty repwas.
     */
    private function get_empty_and_nonempty_repwas(&$empty_repwas, &$non_empty_repwas){
            //get all defwas
            $defwas = $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_defwas();

            // foreach defwa
            foreach ($defwas->getList() as $defwa){
                // get json format of current defwa id
                $defwa_without_spaces = str_replace(' ','_',$defwa->get_id());
                $request_defwa_val = $_REQUEST[$defwa_without_spaces];
                // if current row repwa is empty
                if (strcmp($request_defwa_val, "") === 0){
                    //add the repwa to the empty repwas
                    $empty_repwas[$defwa->defwa_id] = $_REQUEST[$defwa_without_spaces];
                }
                // else - if current row repwa is non empty
                else {
                    //add the repwa to the non empty repwas
                    $non_empty_repwas[$defwa->get_id()] = $_REQUEST[$defwa_without_spaces];
                }
            }
    }
    
    
    
    
}

?>
