<?php

/*
 * Defines
 */
define('pw_edit_mode', pw_repwas . '/edit_mode' );
define('pw_replaceables', pw_repwas . '/replaceables' );
define('pw_replaceables_basic', pw_replaceables . '/basic' );
define('pw_replaceables_categories', pw_replaceables . '/categories' );
define('pw_replaceables_page_templates', pw_replaceables . '/page_templates' );
define('pw_replaceables_posts', pw_replaceables . '/posts' );
define('pw_replaceables_tags', pw_replaceables . '/tags' );

// urls

/*
 * Requires
 */
require_once(pw_repwas . '/RepwasAjaxHandler.php');
require_once(pw_repwas . '/gui/RepwasGuiManager.php');
require_once(pw_repwas . '/Repwa.php');
require_once(pw_repwas . '/RepwaList.php');

// basic
require_once(pw_replaceables_basic . '/ReplaceableInterface.php');
require_once(pw_replaceables_basic . '/BasicReplaceable.php');
require_once(pw_replaceables_basic . '/ReplaceableList.php');
require_once(pw_replaceables_basic . '/ReplaceableInterface.php');

// pages and posts
require_once(pw_replaceables_posts . '/PwPost.php');
require_once(pw_replaceables_posts . '/PwPostList.php');
require_once(pw_replaceables_posts . '/RepwasPagesPostsManager.php');

// categories
require_once(pw_replaceables_categories . '/PwCategory.php');
require_once(pw_replaceables_categories . '/PwCategoryList.php');
require_once(pw_replaceables_categories . '/RepwasCategoriesManager.php');

// tags
require_once(pw_replaceables_tags . '/PwTag.php');
require_once(pw_replaceables_tags . '/PwTagList.php');
require_once(pw_replaceables_tags . '/RepwasTagsManager.php');

// page templates
require_once(pw_replaceables_page_templates . '/PwPageTemplate.php');
require_once(pw_replaceables_page_templates . '/PwPageTemplateList.php');
require_once(pw_replaceables_page_templates . '/RepwasPageTemplatesManager.php');

// repwas edit mode
require_once(pw_edit_mode . '/RepwasEditMode.php');

// check if class name exists
if(!class_exists('RepwasManager')){
    
    /**
     * RepwasManager is in charge of replacements management, including:
     *      (1) replacements insertion
     *      (2) replacements deletion
     *      (3) replacements update
     *      (4) replace custom widget areas
     *
     * @author Jonathan
     */
    class RepwasManager {
        private $repsAjaxHandler;   // handles ajax requests
        private $repwasGuiManager;    // manages repwas gui
        private $repwasEditMode; // repwasEditMode
        private $repwasPagesPostsManager;   // manager of pages and posts
        private $repwasCategoriesManager;   //manger of categories
        private $repwasTagsManager;   //manager of tags
        private $repwasPageTemplatesManager;    // manager of page templates


/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * Constructor of RepwasManager. 
         * @param none
         * @return RepwasManager new RepwasManager instance
         */
        public function __construct() {
            $this->repsAjaxHandler = new RepwasAjaxHandler();
            $this->repwasGuiManager = new RepwasGuiManager();
            $this->repwasEditMode = new RepwasEditMode();
            $this->repwasPagesPostsManager = new RepwasPagesPostsManager();
            $this->repwasCategoriesManager = new RepwasCategoriesManager();
            $this->repwasTagsManager = new RepwasTagsManager();
            $this->repwasPageTemplatesManager= new RepwasPageTemplatesManager();
            
        } // end RepwasManager Constructor

/* ---------------------------------------getters---------------------------------------------------------------------------------------*/
        
        function get_repwasGuiManager (){
            return $this->repwasGuiManager;
        }
        
        function get_repwasEditMode () {
            return $this->repwasEditMode;
        }
        
        function get_repwasPagesPostsManager (){
            return $this->repwasPagesPostsManager;
        }
        
        function get_repwasCategoriesManager () {
            return $this->repwasCategoriesManager;
        }
        
        function get_repwasTagsManager () {
            return $this->repwasTagsManager;
        }
        
        function get_repwasPageTemplatesManager () {
            return $this->repwasPageTemplatesManager;
        }
        
/* ---------------------------------------end getters----------------------------------------------------------------------------------*/
/* ---------------------------------------setters----------------------------------------------------------------------------------------*/
        
        function set_repwasGuiManager ($value) {
            $this->repwasGuiManager = $value;
        }
        
        function set_repwasEditMode ($value) {
            $this->repwasEditMode = $value;
        }
        
        function set_repwasPagesPostsManager ($value) {
            $this->repwasPagesPostsManager = $value;
        }
        
        function set_repwasCategoriesManager ($value) {
            $this->repwasCategoriesManager = $value;
        }

        function set_repwasTagsManager ($value) {
            $this->repwasTagsManager = $value;
        }
        
        function set_repwasPageTemplatesManager ($value) {
            $this->repwasPageTemplatesManager = $value;
        }
        
/* ---------------------------------------end setters----------------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * inserts a repwa to the db
         * @param Repwa $repwa
         */
        public function insertRepwaToDb ($repwa) {
            DbFunctionsLite::insertRepwa($repwa);
        }   // end function insertRepwa
        
        
/* -------------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * inserts a repwa to the db or update it if it already exists
         * @param Repwa $repwa
         */
        public function insertOrUpdateRepwaInDb ($repwa) {
            DbFunctionsLite::insertOrUpdateRepwa($repwa);
        }   // end function insertRepwa
        
        
    }   // end class RepwasManager
}   // end if class not exist

/*
 * TODO: load table when page is loaded rather than on ajax request
 */
?>
