<?php

if(!class_exists('WaReplacer')){
    
    /**
     * WaReplacer offers services of actually replacing the widget areas
     *
     * @author Jonathan
     */
    class WaReplacer {
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
    /**
     * constructor of WaReplacer
     */
        public function __construct() {
            /* filters */
            //make sidebars swap as defined in the postmeta options
            add_filter('sidebars_widgets', array($this, 'je_sb_replace_widget_areas'), 10);
            
        } // end WaReplacer Constructor
        
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * Replaces default widget areas with custom widget areas
         * @param array $sidebars_widgets 
         *      array of current registered widget areas
         * @return array updated widget areas with defwas ids as keys and widgets as values
         */
        function je_sb_replace_widget_areas($sidebars_widgets) {

            // get pw replaceable that is being viewed
            $replaceable = $this->getBeingViewedReplaceable();

            if ($replaceable) {
                $this->checkIfHasRepwasAndReplace ($sidebars_widgets, $replaceable);
            }
            
            return $sidebars_widgets;

        }   // end function je_sb_replace_widget_areas
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * gets being viewed replaceable object
         * @global type $post
         * @global type $post
         * @param none
         * @return BasicReplaceable
         */
        private function getBeingViewedReplaceable() {
            $repwa_type = $this->choose_overriding_repwa_type();
            switch ($repwa_type) {
                case "page":
                    global $post;
                    return $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasPagesPostsManager()->get_pwPages()->getById($post->ID);
                    break;
                case "post":
                    global $post;
                    return $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasPagesPostsManager()->get_pwPosts()->getById($post->ID);
                    break;
                case "category":
                    $curr_category_info = get_the_category();
                    $curr_category_info = $curr_category_info[0];
                    $curr_category_id = $curr_category_info->cat_ID;
                    return $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasCategoriesManager()->get_pwCategories()->getById($curr_category_id);
                    break;
                case "tag":
                    $all_curr_tags_info = get_the_tags();  // there should be only one tag returned (because it's a tag page)
                    $curr_tag = reset ($all_curr_tags_info);
                    $curr_tag_id = $curr_tag->term_id;
                    return $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasTagsManager()->get_pwTags()->getById($curr_tag_id);
                    break;
                case ($repwa_type == "blog_page" ||  
                    $repwa_type == "404_page" ||  
                    $repwa_type == "search_results_page" ||
                    $repwa_type == "date_archive"):
                    return $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasPageTemplatesManager()->get_pwPageTemplates()->getByType($repwa_type);

                    break;
                default:
                break;
            }
        }
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * Returns a string telling the replaceable element type which is currently viewed by the user
         * @param none
         * @return string replaceable element type. can be one of the values listed below: 
         *      "post" - if the replaceable element is a post
         *      "page" - if the replaceable element is a page
         *      "category" - if the replaceable element is a category
         *      "tag" - if the replaceable element is a tag
         *      "blog_page" - if the replaceable element is a blog page
         *      "404_page" - if the replaceable element is a 404
         *      "search_results_page" - if the replaceable element is a search result page
         *      "date_archive" - if the replaceable element is a date archive
         */
        function choose_overriding_repwa_type() {
            if (is_single())    { return "post"; }
            else if (is_page()) { return "page"; }
            else if(is_category())  { return "category"; }
            else if(is_tag())  { return "tag"; }
            else if(is_home())  { return "blog_page"; }
            else if(is_404())  { return "404_page"; }
            else if(is_search())  { return "search_results_page"; }
            else if(is_date())  { return "date_archive"; }
        }   // end function choose_overriding_repwa_type

        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * checks if replaceable has repwas. 
         * If so, replaces defwas widgets with custwas widgets using swapRepwas
         * @param array $sidebars_widgets
         * @param BasicReplaceable $replaceable
         */
        private function checkIfHasRepwasAndReplace(&$sidebars_widgets, $replaceable) {

            //check if there are replacements for the current replaceable
            $replaceableRepwas = $replaceable->get_repwaList()->getList();

            //go through each replacement widget area and swap default sb and custom sb
            $sidebars_widgets = $this->swapRepwas($sidebars_widgets, $replaceableRepwas);
                
        }   // end function checkIfHasRepwasAndReplace

/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * swaps defwas with custwas according to the replacements provided in $replacements
         * @param array $sidebars_widgets 
         *      array of widget areas and their widgets
         * @param array $replacements Description
         *      array of replacement record with the following attributes
         * @return array
         *      array of updated widget areas: with defwas as keys widgets as values
         */
        private function swapRepwas($sidebars_widgets, $replacements ){
            //go through each replacement widget area and swap default sb and custom sb
            foreach($replacements as $replacement){
                    $default_sidebar = $replacement->get_defwa()->get_id();
                    $custom_sidebar = $replacement->get_custwa()->get_name();
                    $sidebars_widgets[$default_sidebar] = $sidebars_widgets[$custom_sidebar];
            }
            return $sidebars_widgets;
        }   // end function
    }   // end class WaReplacer
}   // end if class WaReplacer not exist
?>
