<?php

require_once ( pw_classes . '/repwas/gui/LogicRepwasTableHeader.php' );


if(!class_exists('LogicRepwasTable')){
    /**
     * logicRepwasTable is an object containing the data of a replacements control table
     *
     * @author Jonathan
     */
    class LogicRepwasTable {
        private $header;
        private $data;  // table data

        /**
         * Constructor of LogicRepwasTable
         * creates a new table with the specified metadata, and with data
         * filled according to the given $repType parameter
         * @param string $repType
         *      the replaceable type. Can be one of the following: 
         *      page, post, category, tag, page_template
         * @param DefwaList $defList
         *      all theme's default widget areas
         *      indicates whether there's a boolean column
         */
        public function __construct ($repType, $defList, $searchedText, $editedElement) {

            // initialize header
            $this->header = new LogicRepwasTableHeader($repType, $defList, $searchedText, $editedElement);

            // get appropriate replaceable table data
            $this->data = $this->getReplaceableTableData( $searchedText, $editedElement );
        }


        /**
         * Constructor of LogicRepwasTable with another logic repwas table
         * @param LogicRepwasTable $lgcRepwasData
         *      a LogicRepwasTable object
         */
        public static function constructWithLogicRepwasTable ($lgcRepwasData) {
            $this->header = $lgcRepwasData->header;
            $this->data = $lgcRepwasData->data;
        }


    /* ---------------------------------------getters---------------------------------------------------------------------------------------*/
        public function getHeader () {
            return $this->header;
        }
        
        public function getData () {
            return $this->data;
        }

    /* ---------------------------------------end getters----------------------------------------------------------------------------------*/
    /* ---------------------------------------setters----------------------------------------------------------------------------------------*/
        public function setHeader ($value) {
            $this->header = $value;
        }
        
        public function setData ($value) {
            $this->data = $value;
        }

    /* ---------------------------------------end setters----------------------------------------------------------------------------------*/

        private function getReplaceableTableData( $searchedText, $editedElement ) {
            // get repleaceable type
            $repType = $this->getHeader()->getRepType();
            $fullReplaceablesData = NULL;
            $returnedReplaceables = NULL;
            
            // get appropriate replaceable data
            switch ($repType) {
                case "page":
                    $fullReplaceablesData = $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasPagesPostsManager()->get_pwPages();
                    $returnedReplaceables = new PwPostList();
                    break;
                case "post":
                    $fullReplaceablesData = $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasPagesPostsManager()->get_pwPosts();
                    $returnedReplaceables = new PwPostList();
                    break;
                case "category":
                    $fullReplaceablesData = $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasCategoriesManager()->get_pwCategories();
                    $returnedReplaceables = new PwCategorytList();
                    break;
                case "tag":
                    $fullReplaceablesData = $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasTagsManager()->get_pwTags();
                    $returnedReplaceables = new PwTagList();
                    break;
                case "page_template":
                    $fullReplaceablesData = $GLOBALS['mainPwObj']->get_je_sb_repwas_manager()->get_repwasPageTemplatesManager()->get_pwPageTemplates();
                    $returnedReplaceables = new PwPageTemplateList();
                    break;
                default:
                    break;
            }   // end switch

            if ( $editedElement > 0 ) { 
                $returnedReplaceables->add($fullReplaceablesData->getById($editedElement));
            }
            
            else if (!empty ($searchedText)) {
                $returnedReplaceables = $fullReplaceablesData->getCloseTitleFilteredList($searchedText);
            }
            else {
                $returnedReplaceables = $fullReplaceablesData;
            }
            return $returnedReplaceables;
        }   // end function generateTableData
    }   // end  class LogicRepwasTable
}   // end if class not exist

?>
