<?php

/**
 * RepwasGuiManager includes all gui components used for the repwas feature
 * and an output function for printing a replacements control interface
 *
 * @author Jonathan
 */

require_once(pw_repwas . '/gui/RepwasTable.php');

class RepwasGuiManager extends GuiManager{
    private $currTable;            // current table being viewed (visual component)
    private $custwasDropdown;    // the custwas dropdown list
    
    /*
     * 
     * all RepwasGuiManager members are set only on ajax request. Until then, they are NULL.
     */
    public function __construct() {
        $this->currTable = NULL;
        $this->custwasDropdown = NULL;
        
        /* actions */
        // handle change direction of jeasyui control table form submission
        add_action( 'plugins_loaded', array ($this, 'hanle_change_direction_form_submission') );
    }
    
/* ---------------------------------------getters---------------------------------------------------------------------------------------*/
    function get_currTable () {
        return $this->currTable;
    }
    
    function get_custwasDropdown (){
        return $this->custwasDropdown;
    }

/* ---------------------------------------end getters----------------------------------------------------------------------------------*/

/* ---------------------------------------setters----------------------------------------------------------------------------------------*/
    function set_currTable ($value){
        $this->currTable = $value;
    }
    
    function set_custwasDropdown ($value) {
        $this->custwasDropdown = $value;
    }
    
/* ---------------------------------------end setters----------------------------------------------------------------------------------*/

/* ---------------------------------------------------------Pages----------------------------------------------------------------------*/
    /**
     * outputs "Pages" content, including:
     *  (1) form to adjust pages table direction
     *  (2) pages jquery table
     * @param none 
     * @return void 
     */
    function outputPagesContent() {
            $this->je_sb_do_page_output(__('Power Widgets Pages Control', 'PWL'));
            $this->je_sb_print_CSControl_interface('page');
    }
   
/* ---------------------------------------------------------End Pages-----------------------------------------------------------------*/
/* ---------------------------------------------------------Posts----------------------------------------------------------------------*/
    /**
     * outputs "Posts" content, including:
     *  (1) form to adjust posts table direction
     *  (2) posts jquery table
     * @param none
     * @return void 
     */
    function outputPostsContent() {
            $this->je_sb_do_page_output(__('Power Widgets Posts Control', 'PWL'));
            $this->je_sb_print_CSControl_interface('post');
    }
/* ---------------------------------------------------------End Posts-----------------------------------------------------------------*/
/* ---------------------------------------------------------Categories-----------------------------------------------------------------*/
    /**
     * outputs "Categories" content, including:
     *  (1) form to adjust categories table direction
     *  (2) categories jquery table
     * @param none
     * @return void
     */
    function outputCategoriesContent() {
            $this->je_sb_do_page_output(__('Power Widgets Categories Control', 'PWL'));
            $this->je_sb_print_CSControl_interface('category');
    }
/* ---------------------------------------------------------End Categories----------------------------------------------------------*/
/* ---------------------------------------------------------Tags-----------------------------------------------------------------*/
    /**
     * outputs "Tags" content, including:
     *  (1) form to adjust tags table direction
     *  (2) tags jquery table
     * @param  none 
     * @return void 
     */
    function outputTagsContent() {
            $this->je_sb_do_page_output(__('Power Widgets Tags Control', 'PWL'));
            $this->je_sb_print_CSControl_interface('tag');
    }
/* ---------------------------------------------------------End Tags-----------------------------------------------------------------*/
/* ---------------------------------------------------------Page Templates--------------------------------------------------------*/
    /**
     * outputs "Page Templates" content, including:
     *  (1) form to adjust page templates table direction
     *  (2) page templates jquery table
     * @param  none 
     * @return void 
     */
    function outputPageTemplatesContent() {
            $this->je_sb_do_page_output(__('Power Widgets Page Templates Control', 'PWL'));
            $this->je_sb_print_CSControl_interface('page_template');
    }
/* ---------------------------------------------------------End Page Templates-----------------------------------------------------*/
    
    
/* --------------------------------------------------------------------------------------------------------------------------------------------*/
    /**
     * prints the interface of all sections which use the jquery replacements table.
     * First, the function prints the direction adjustment form
     * Secondly, the fhe function prints all the gui of the table. While doing so, 
     * the function differeniates between different replaceable elements: pages, posts, categories, tags, 
     * page templates and patterns. Additionally in case this table is used in page/post edit mode the function
     * builds the table in such a way that it contains only 1 record - the relevant page/post that is in edit mode.
     * @param string $rep_type Description
     *      the replacmeable element type (page, post, category, tag, page template, pattern)
     * @param string $edited_post 
     *      $edited_post (optional) - if replacements control table is combined in edit page/post mode then 
     *      it should  the relevant edited page/post id which is represented by $edited_post.
     * @return void
     */
    function je_sb_print_CSControl_interface($rep_type, $edited_post = NULL) {
            $direction = get_option('pw_dir', NULL);
            $direction_msg = "";
            if ($direction === 'ltr')   {$direction_msg = __("left to right", 'PWL');}
            else {$direction_msg = __('right to left', 'PWL');}
?>
            <form action="<?php $_SERVER['PHP_SELF'];?>" method="post">
               <p> <?php _e('At the moment, the plugin is adjusted to', 'PWL'); echo "<b>" . $direction_msg . "</b>"; _e('direction. To change it Click the button below.', 'PWL') ?> </p>
               <input name = "form_type" type="hidden" value="change_direction"></input>
               <p><input name="change_direction_submit" type="submit" value="<?php _e('Adjust Direction', 'PWL'); ?>" class="button-primary"/></p>
            </form>

            <div id="site-url" style="display:none"><?php echo site_url();?></div>
            <div id="admin-url" style="display:none"><?php echo admin_url();?></div>
            <div id="pw-scripts" style="display:none"><?php echo pw_scripts; ?></div>
            <div id="current-theme" style="display:none"><?php echo "current_theme=" . wp_get_theme();?></div>
            <div id="rep-type" style="display:none"><?php echo "rep_type=" . $rep_type; ?></div>
            <div id="edited-post" style="display:none"><?php echo $edited_post; ?></div>
            <br/>
            <h3><?php _e('Replace a Default Widget Area with a Custom Widget Area', 'PWL'); ?></h3>

            <div style="width:97%;" >
            <table id="dg" title="<?php _e('Power Widgets Control', 'PWL'); ?>" 
                    toolbar="#toolbar" pagination="true" idField="replaceable_element_id"
                    rownumbers="true" fitColumns="false" singleSelect="true">
                                                <thead frozen="true">
                                                    <tr>
                                                        <?php 
                                                            $get_custwas_url = admin_url(). "admin-ajax.php?action=get_custwas_dropdown";
                                                            
                                                            if ($rep_type === 'page' ){
                                                                echo "<th field=\"replaceable_element\" formatter=\"formatReplaceableElement\">"; _e('Page', 'PWL'); echo "</th>";
                                                            }
                                                            else if ($rep_type === 'post' ){
                                                                echo "<th field=\"replaceable_element\" formatter=\"formatReplaceableElement\">"; _e('Post', 'PWL'); echo "</th>";
                                                            }
                                                            else if ($rep_type === 'category'){
                                                                echo "<th field=\"replaceable_element\">"; _e('Category', 'PWL'); echo "</th>";
                                                            }
                                                            
                                                            else if ($rep_type === 'tag'){
                                                                echo "<th field=\"replaceable_element\">"; _e("Tag", 'PWL'); echo "</th>";
                                                            }
                                                            
                                                            else if ($rep_type === 'page_template'){
                                                                echo "<th field=\"replaceable_element\">"; _e("Page Template", 'PWL'); echo "</th>";
                                                            }

                                                        ?>

                                                    </tr>
                                                </thead>
                                                <thead>
                                                    <tr>
                                                        <?php 
                                                            $theme_defwas = $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_defwas();

                                                            // set cuswas_rs value here in order to save multiple db calls
                                                            foreach ($theme_defwas->getList() as $defwa){
                                                                $defwa_json_id = str_replace(' ','_',$defwa->get_id());
                                                                $defwa_name = $defwa->get_name();
                                                                echo "<th field=\"". $defwa_json_id ."\" editor=\"{type:'combobox',options:
                                                                            {editable:false,
                                                                            panelHeight:'auto',
                                                                            valueField:'id',
                                                                            textField:'text',
                                                                            formatter:function(row){
                                                                                return row.text=='' ? '&nbsp;' : row.text;
                                                                            },
                                                                            url:'" . $get_custwas_url ."'}}\">" . $defwa_name ."</th>";
                                                            }

                                                        ?>
                                                    </tr>

                                                </thead>    
                            </table>
                </div>
                <div id="toolbar">
                        <a href="#" class="easyui-linkbutton" iconCls="icon-save" plain="true" onclick="javascript:jQuery('#dg').edatagrid('saveRow')"><?php _e('Save', 'PWL')?></a>
                        <a href="#" class="easyui-linkbutton" iconCls="icon-undo" plain="true" onclick="javascript:jQuery('#dg').edatagrid('cancelRow')"><?php _e('Cancel', 'PWL')?></a>
                        <span>
                            <?php 
                             if (!$edited_post){
                           /*     
                             	if ($rep_type === "page_template")  {echo "Page Template";}
                                else    {echo ucfirst ($rep_type);} 
                            */
                                 echo $this->getTranslatedReptype($rep_type);
                                
                            ?>
                        </span>
                            <input id="replaceable_element" style="line-height:26px;border:1px solid #ccc">   
                            <a href="#" class="easyui-linkbutton" plain="true" onclick="doSearch()"><?php _e('Search', 'PWL')?></a>  
                        <?php
                            }
                        ?>
                </div>

<?php
    } //end function je_sb_print_CSControl_interface
    
    
    /* --------------------------------------------------------------------------------------------------------------------------------------------*/
    /**
     * 
     * returns translated reptype
     * @param unknown_type $reptype
     */
    function getTranslatedReptype($reptype){
        switch($reptype){
            case "page": 
                return __('Page', 'PWL');
                break;
            case "post": 
                return __('Post', 'PWL');
                break;
            case "category": 
                return __('Category', 'PWL');
                break;
            case "tag": 
                return __('Tag', 'PWL');
                break;
            case "page_template": 
                return __('Page Template', 'PWL');
                break;
        }
    }
    
    /* --------------------------------------------------------------------------------------------------------------------------------------------*/
    /**
     * handles change direction of jeasyui table form submission
     * @param none
     * @return void
     */
    function hanle_change_direction_form_submission() {
        //if a change direction form has been sent
        if ( $_SERVER['REQUEST_METHOD'] === "POST" &&
                isset($_POST['change_direction_submit']) &&
                $_POST['change_direction_submit']
        )
            
        {
            $direction = get_option('pw_dir', NULL);
            if ($direction === 'ltr'){
                update_option('pw_dir', 'rtl');
            }
            else{
                update_option('pw_dir', 'ltr');
            }
        }
    } // end function je_sb_handle_form_submission
    
    
    /**
     * outputs theme's custwa array into select box
     * @static
     * @param array $theme_custwa_result Description
     *      array contatining all theme custwas
     * @param string $select_box_name 
     *      the select box name attribute
     * @return string 
     *      string containing select box html instructions
     */
    static function outputWaListToSelectBox($waList, $select_box_name,  $firstOptionVal = FALSE){
            $select_box = "<select name='" . $select_box_name. "' style='width: 200px'>";
            
            // if first empty is true - create an empty option
            if ($firstOptionVal)    {
                $select_box .= "<option value='pwEmptyOption'>$firstOptionVal</option>";
            }
            if ($waList){
                foreach ($waList as $wa) {
                    $select_box .= "<option value='" . $wa->get_id() . "'>" .
                        $wa->get_name() . "</option>";
                }
            }
            $select_box.= "</select>";// Closing of select box

            return $select_box;
    }
    
    
} //end class RepwasGuiManager

?>
