<?php

/**
 * VisualRepwasTable represents a visual replacements control table.
 * It is composed of visual elements that can be be displayed by echoing them
 * It also provides methods to display these elements
 *
 * @author Jonathan
 */
class RepwasTable {
    private $logicTable;  // the logical table
    private $offset; // the record number from which the table starts
    private $rowsNum;   // table rows number being viewed
    private $jeasyuiTableFormat;   // the visual table component
    
    const OFFSET_TO_DEFWA = 3;
    
    /**
     * Constructor of VisualRepwasTable
     * should be called **only** after the jeasyui ajax hook
     * gets a logic table and generates a visual table component out of it
     * @param type $logicTable
     * @param int $offset
     *      offset (in records number) from the beginning of the table
     * @param int $rows
     *      number of rows to be viewed
     */
    public function __construct($logicTable, $offset, $rowsNum) {
            
            // members initialization
            $this->logicTable = $logicTable;
            $this->offset = $offset;
            $this->rowsNum = $rowsNum;
            $this->jeasyuiTableFormat = $this->generateJeasyuiTableFromLogicalTable($logicTable);
            
    }   // end of Constructor
    
    /* ---------------------------------------getters---------------------------------------------------------------------------------------*/
        public function getLogicTable () {
            return $this->logicTable;
        }
        
        public function getOffset () {
            return $this->offset;
        }
        
        public function getRowsNum () {
            return $this->rowsNum;
        }
        
        public function getJeasyuiTableFormat () {
            return $this->jeasyuiTableFormat;
        }

    /* ---------------------------------------end getters----------------------------------------------------------------------------------*/
    /* ---------------------------------------setters----------------------------------------------------------------------------------------*/
        public function setLogicTable ($value) {
            $this->logicTable = $value;
        }
        
        public function setOffset ($value) {
            $this->offset = $value;
        }
        
        public function setRowsNum ($value) {
            $this->rowsNum = $value;
        }
        
        public function setJeasyuiTableFormat ($value) {
            $this->jeasyuiTableFormat = $value;
        }

    /* ---------------------------------------end setters----------------------------------------------------------------------------------*/
    
    private function generateJeasyuiTableFromLogicalTable( $logicTable) {
            $table = $rows_objs_arr = NULL;

            // initialize table
            $totalRowsNum = count ( $logicTable->getData()->getList() );
            $this->initialize_table ($table, $rows_objs_arr, $totalRowsNum);

            // if there are elements
            if ($totalRowsNum > 0) {

                // cut only the relevant elements
                $replaceables = array_slice ( $logicTable->getData()->getList() , $this->offset, $this->rowsNum, true );

                // go through all elements
                foreach ($replaceables as $replaceable) {

                    //build row object
                    $row_obj = new stdClass;
                    $row_obj->replaceable_element_id = $replaceable->get_id();     // id
                    $row_obj->replaceable_element = $replaceable->get_title();      // title
                    
                    $this->add_defwas_columns ($row_obj, $replaceable);  //defwas

                    // push the current built row object to the rows objects array
                    array_push($rows_objs_arr, $row_obj);
                }
            }

            // attach rows array to table
            $this->attach_rows_to_table ($table, $rows_objs_arr);

            return $table;
    }   // end function generateJeasyuiTableFromLogicalTable
    


    
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
    
    private function initialize_table (&$table, &$rows_objs_arr, $totalRowsNum){
        $table = array();
        $table["total"] = $totalRowsNum;
        $rows_objs_arr = array();
    }

    private function attach_rows_to_table (&$table, $rows_objs_arr) {
        $table["rows"] = $rows_objs_arr;
    }

    private function add_defwas_columns  (&$row_obj, $replaceable ) {
        // loop throgh all defwas
        foreach ($replaceable->get_repwaList()->getList() as $repwa){
            // get replacement defwa id (spaces (' ') are replaced with underscores ('_')
            $defwaId = str_replace(' ','_',$repwa->get_defwa()->get_id());
            // get replacement custwa name
            $custwaName = $repwa->get_custwa()->get_name();
            
            // if there's no replacement for this defwa
            if ( !$custwaName ) {   $row_obj->$defwaId = '';   }
            // else (there's a replacement for this defwa)
            else {  $row_obj->$defwaId = $custwaName;  }
            }
    }
            
    /**
     * Echos table gui component
     * @param none
     * @return void
     */
    public function __toString() {
            $json_format_table = json_encode($this->jeasyuiTableFormat);
            return $json_format_table;
    }   // end function __toString
}

?>
