<?php

if(!class_exists('RepwasCategoriesManager')){
    
    /**
     * RepwasCategoriesManager is in charge of categories replacements management, including:
     *      (1) categories replacements insertion
     *      (2) categories replacements deletion
     *      (3) categories replacements update
     *
     * @author Jonathan
     */
    class RepwasCategoriesManager {
        
        private $pwCategories;  // PowerWidgets categories
        
        /**
         * Constructor of RepwasCategoriesManager
         */
        public function __construct() {
                /* members */
                $this->pwCategories = new PwCategorytList();

                /* actions */

                //get categories from db
                add_action('widgets_init', array ($this, 'generatePwCategoriesFromDb'), 11);

                // delete also cateory's replacements on category delete
                add_action('delete_category' , array($this,'deleteCategoryReplacements'), 10);
            
        }   // end Constructor
        
/* ---------------------------------------------------------------getters-----------------------------------------------------------------*/
        function get_pwCategories () {
            return $this->pwCategories;
        }
/* -------------------------------------------------------end getters------------------------------------------------------------------*/
/* --------------------------------------------------------------setters-----------------------------------------------------------------*/
        function set_pwCategories ($value) {
            $this->pwCategories = $value;
        }
        
/* -------------------------------------------------------end setters------------------------------------------------------------------*/
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * gets PwCategoryList from db
         * @param none
         * @return PwCategoryList 
         *      list of categories
         */
        public function generatePwCategoriesFromDb () {
            // initialize PwCategories list
            $returnedPwCategoriesList = new PwCategorytList();
            $theme = wp_get_theme();
            
            // get categories from db
            $dbCategories = $this->getCategoriesFromDb();
            
            // if there are categories
            if ($dbCategories) {
                // for each db category
                foreach ($dbCategories as $dbCategory) {

                    // get category repwas from db
                    $categoryDbRepwas = $this->getCategoryRepwas($dbCategory->cat_ID, $theme);

                    // transfer the db repwas to RepwaList
                    $categoryRepwaList = RepwaList::dbRepwasToRepwaList(
                            $categoryDbRepwas, 
                            array ( "repwa_id", "defwa_id", "custwa_name", "repwa_theme", "repwa_element_id", "repwa_type" ),
                            $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_defwas(),
                            $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_custwas()
                    );

                    // build a PwCategory object with db category and the RepwaList
                    $pwCategory = new PwCategory($dbCategory, $categoryRepwaList);
                             

                    // add the category to the list
                   $returnedPwCategoriesList->add($pwCategory);
                }
            }
            
            $this->set_pwCategories($returnedPwCategoriesList);
            return $returnedPwCategoriesList;
        }   // end function buildPwCategoriesMember
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * returns all categories from db
         * @param none
         */
        public function getCategoriesFromDb () {                
                // get categories
                $args = array(
                        'type'                     => '',
                        'child_of'                 => 0,
                        'parent'                   => '',
                        'orderby'                  => 'name',
                        'order'                    => 'ASC',
                        'hide_empty'               => 0,
                        'hierarchical'             => 1,
                        'exclude'                  => '',
                        'include'                  => '',
                        'number'                   => '',
                        'taxonomy'                 => 'category',
                        'pad_counts'               => false 
                    );
                
                $dbCategories = get_categories( $args );
                
                return $dbCategories;
        }   // end function getPwPagesFromDb
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * gets categories repwas from db
         * @param int $pId
         *      category id
         * @param WP_Theme $theme
         *      repwas theme
         * @return array
         */
        private function getCategoryRepwas ($category_id, $theme) {
            return DbFunctionsLite::getCategoryRepwasByTheme($theme, $category_id);
        }   // end function getCategoryRepwas
        
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * deletes all the replacements of category $cid
         * @param int $cid 
         *      category id
         * @return boolean
         *       true if category was successfully deleted and false otherwise.
         */
        function deleteCategoryReplacements($cid) {
                // delete all category repwas
                DbFunctionsLite::delete_element_repwas($cid, 'category');
                return true;
            
            return false;
        } // end function deleteCategoryReplacements
        
    }   // end class RepwasCategoriesManager
}   // end if class RepwasCategoriesManager exists



?>
