<?php

if(!class_exists('RepwasPageTemplatesManager')){
    
    /**
     * RepwasPageTemplatesManager is in charge of pageTemplates replacements management, including:
     *      (1) pageTemplates replacements insertion
     *      (2) pageTemplates replacements deletion
     *      (3) pageTemplates replacements update
     *
     * @author Jonathan
     */
    class RepwasPageTemplatesManager {
        
        private $pwPageTemplates;  // PowerWidgets pageTemplates
        
        /**
         * Constructor of RepwasPageTemplatesManager
         */
        public function __construct() {
                /* members */
                $this->pwPageTemplates = new PwPageTemplateList();

                /* actions */

                //get page templates from db
                add_action('widgets_init', array ($this, 'generatePwPageTemplates'), 11);
        }   // end Constructor
        
/* ---------------------------------------------------------------getters-----------------------------------------------------------------*/
        function get_pwPageTemplates () {
            return $this->pwPageTemplates;
        }
/* -------------------------------------------------------end getters------------------------------------------------------------------*/
/* --------------------------------------------------------------setters-----------------------------------------------------------------*/
        function set_pwPageTemplates ($value) {
            $this->pwPageTemplates = $value;
        }
        
/* -------------------------------------------------------end setters------------------------------------------------------------------*/
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * gets PwCategoryList from db
         * @param none
         * @return PwCategoryList 
         *      list of pageTemplates
         */
        public function generatePwPageTemplates () {

                // initialize PwPageTemplates list
                $returnedPwPageTemplatesList = new PwPageTemplateList();

                // build all page templates
                $returnedPwPageTemplatesList->add( new PwPageTemplate( 1, "blog_page", "Blog Page" ) );
                $returnedPwPageTemplatesList->add( new PwPageTemplate( 2, "404_page", "404 Page" ) );
                $returnedPwPageTemplatesList->add( new PwPageTemplate( 3, "search_results_page", "Search Results Page" ) );
                $returnedPwPageTemplatesList->add( new PwPageTemplate (4, "date_archive", "Date Archive") );

                foreach ($returnedPwPageTemplatesList->getList() as $pTemplate) {

                    $theme = wp_get_theme();
                    // set page template repwas
                    $pTemplate->set_repwaList (
                            $this->getPageTemplateRepwas(
                                    $pTemplate->get_id(), $pTemplate->getType(), $theme
                            )
                        );
                }
                $this->set_pwPageTemplates($returnedPwPageTemplatesList);
                return $returnedPwPageTemplatesList;

        }   // end function buildPwPageTemplatesMember
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * gets page templates from db
         * @param int $pId
         *      pageTemplate id
         * @param WP_Theme $theme
         *      repwas theme
         * @return array
         */
        private function getPageTemplateRepwas ($ptId, $ptType, $theme) {
            $dbRepwas = DbFunctionsLite::getPageTemplateRepwasByTheme($theme, $ptId, $ptType);
            
            // transfer the db repwas to RepwaList
            return RepwaList::dbRepwasToRepwaList(
                    $dbRepwas, 
                    array ( "repwa_id", "defwa_id", "custwa_name", "repwa_theme", "repwa_element_id", "repwa_type" ),
                    $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_defwas(),
                    $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_custwas()
            );
            
        }   // end function getCategoryRepwas
        
    }   // end class RepwasPageTemplatesManager
}   // end if class RepwasPageTemplatesManager exists



?>
