<?php

if(!class_exists('RepwasPagesPostsManager')){
    
    /**
     * RepwasPagesAndPostsManager is in charge of pages and posts replacements management, including:
     *      (1) pages and posts replacements insertion
     *      (2) pages and posts replacements deletion
     *      (3) pages and posts replacements update
     *
     * @author Jonathan
     */
    class RepwasPagesPostsManager {
        
        private $pwPosts;  // PowerWidgets posts
        private $pwPages;  // PowerWidgets pages
        
        /**
         * Constructor of RepwasPagesAndPostsManager
         */
        public function __construct() {
            /* members */
            $this->pwPosts = new PwPostList();
            $this->pwPages = new PwPostList();
            
            /* actions */
            
            //get posts from db
            add_action('widgets_init', array ($this, 'generatePwPostsAndPagesFromDb'), 11);
            
            // delete also page's or post's replacements on page delete
            add_action('before_delete_post', array($this, 'deletePostReps'), 10);
            
        }   // end Constructor
        
/* ---------------------------------------------------------------getters-----------------------------------------------------------------*/
        function get_pwPosts () {
            return $this->pwPosts;
        }
        
        function get_pwPages () {
            return $this->pwPages;
        }
/* -------------------------------------------------------end getters------------------------------------------------------------------*/
/* --------------------------------------------------------------setters-----------------------------------------------------------------*/
        function set_pwPosts ($value) {
            $this->pwPosts = $value;
        }
        
        function set_pwPages ($value) {
            $this->pwPages = $value;
        }
        
/* -------------------------------------------------------end setters------------------------------------------------------------------*/
        
        public function generatePwPostsAndPagesFromDb() {
            $this->set_pwPosts($this->generatePwPostsFromDb("post"));
            $this->set_pwPages($this->generatePwPostsFromDb("page"));
        }   // end function generatePwPostsAndPagesFromDb
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * builds PwPostsMember using posts and repwas in db
         * @param none
         * @return array 
         *      array of WP_Post objects - each representing a db post
         */
        private function generatePwPostsFromDb ($postType) {
            // initialize PwPosts list
            $returnedPwPostsList = new PwPostList();
            $theme = wp_get_theme();
            
            // get posts from db
            $dbPosts = $this->getPostsFromDb($postType);
            
            // if there are posts
            if ($dbPosts) {
                // for each db post
                foreach ($dbPosts as $dbPost) {

                    // get post repwas from db
                    $postDbRepwas = $this->getPostRepwas($dbPost->ID, $theme);

                    // transfer the db repwas to RepwaList
                    $postRepwaList = RepwaList::dbRepwasToRepwaList(
                            $postDbRepwas, 
                            array ( "repwa_id", "defwa_id", "custwa_name", "repwa_theme", "repwa_element_id", "repwa_type" ),
                            $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_defwas(),
                            $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_custwas()
                    );

                    // build a PwPost object with db post and the RepwaList
                    $pwPost = new PwPost($dbPost, $postRepwaList);
                             

                    // add the post to the list
                   $returnedPwPostsList->add($pwPost);
                }
            }
            
            $postType == "page" ? $this->pwPages = $returnedPwPostsList : $this->pwPosts = $returnedPwPostsList;
            return $returnedPwPostsList;
        }   // end function buildPwPostsMember
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * returns all posts from db
         * @param none
         * @return PrototypeList
         */
        public function getPostsFromDb ($postType) {
                
                // get posts
                $args = array(
	'posts_per_page'  => -1,
	'offset'          => 0,
	'category'        => '',
	'orderby'         => 'title',
	'order'           => 'ASC',
	'include'         => '',
	'exclude'         => '',
	'meta_key'        => '',
	'meta_value'      => '',
	'post_type'       => $postType,
	'post_mime_type'  => '',
	'post_parent'     => '',
	'post_status'     => 'publish',
	'suppress_filters' => true );
                $dbPosts = get_posts($args);
                
                return $dbPosts;
        }   // end function getPwPagesFromDb
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /** 
         * deletes the all the replacements of post $pid
         * @param int $eid element id to be deleted
         * @return boolean 
         *      was post replacements were successfully deleted.
         *      returns true if everything goes o.k and false otherwise
         */
        function deletePostReps($pId) {

                // get post
                $deletedPost = get_post( $pId );
                $pType = $deletedPost->post_type;
                
                // delete page's/post's replacements
                DbFunctionsLite::delete_element_repwas( $pId, $pType);
                return true;
            return false;
            
        } // end function deletePostReps
        
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * gets post repwas from db
         * @param int $pId
         *      post id
         * @param WP_Theme $theme
         *      repwas theme
         * @return array
         */
        private function getPostRepwas ($pId, $theme) {
            return DbFunctionsLite::getPostRepwasByTheme($pId, $theme);
        }   // end function getPostRepwas
        
    }   // end class RepwasPagesAndPostsManager
}   // end if class RepwasPagesAndPostsManager exists



?>
