<?php

if(!class_exists('RepwasTagsManager')){
    
    /**
     * RepwasTagsManager is in charge of tags replacements management, including:
     *      (1) tags replacements insertion
     *      (2) tags replacements deletion
     *      (3) tags replacements update
     *
     * @author Jonathan
     */
    class RepwasTagsManager {
        
        private $pwTags;  // PowerWidgets tags
        
        /**
         * Constructor of RepwasTagsManager
         */
        public function __construct() {
                /* members */
                $this->pwTags = new PwTagList();

                /* actions */

                //get tags from db
                add_action('widgets_init', array ($this, 'generatePwTagsFromDb'), 11);

                // delete tag's replacements on tag delete
                add_action('delete_post_tag' , array($this,'deleteTagReplacements'), 10);
            
        }   // end Constructor
        
/* ---------------------------------------------------------------getters-----------------------------------------------------------------*/
        function get_pwTags () {
            return $this->pwTags;
        }
/* -------------------------------------------------------end getters------------------------------------------------------------------*/
/* --------------------------------------------------------------setters-----------------------------------------------------------------*/
        function set_pwTags ($value) {
            $this->pwTags = $value;
        }
        
/* -------------------------------------------------------end setters------------------------------------------------------------------*/
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * gets PwTagList from db
         * @param none
         * @return PwTagList 
         *      list of tags
         */
        public function generatePwTagsFromDb () {
            // initialize PwTags list
            $returnedPwTagsList = new PwTagList();
            $theme = wp_get_theme();
            
            // get tags from db
            $dbTags = $this->getTagsFromDb();
            
            // if there are tags
            if ($dbTags) {
                // for each db tag
                foreach ($dbTags as $dbTag) {

                    // get tag repwas from db
                    $tagDbRepwas = $this->getTagRepwas($dbTag->term_id, $theme);

                    // transfer the db repwas to RepwaList
                    $tagRepwaList = RepwaList::dbRepwasToRepwaList(
                            $tagDbRepwas, 
                            array ( "repwa_id", "defwa_id", "custwa_name", "repwa_theme", "repwa_element_id", "repwa_type" ),
                            $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_defwas(),
                            $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->get_custwas()
                    );

                    // build a PwTag object with db tag and the RepwaList
                    $pwTag = new PwTag($dbTag, $tagRepwaList);
                             

                    // add the tag to the list
                   $returnedPwTagsList->add($pwTag);
                }
            }
            
            $this->pwTags = $returnedPwTagsList;
            return $returnedPwTagsList;
        }   // end function buildPwTagsMember
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * returns all tags from db
         * @param none
         */
        public function getTagsFromDb () {               
                // get tags
                $args = array(
                        'hide_empty' => false 
                    );
                
                $dbTags = get_tags( $args );
                
                return $dbTags;
        }   // end function getPwPagesFromDb
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * gets tags repwas from db
         * @param int $pId
         *      tag id
         * @param WP_Theme $theme
         *      repwas theme
         * @return array
         */
        private function getTagRepwas ($tag_id, $theme) {
            return DbFunctionsLite::getTagRepwasByTheme($theme, $tag_id);
        }   // end function getTagRepwas
        
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * deletes all tags $tid replacements
         * @param int $tid
         *      tag id
         * @return boolean
         *      true if tag was successfully deleted and false otherwise.
         */
        function deleteTagReplacements($tid) {
                // delete all category repwas
                DbFunctionsLite::delete_element_repwas($tid, 'tag');
                return true;
        }   // end function delete_tag_replacements
        
    }   // end class RepwasTagsManager
}   // end if class RepwasTagsManager exists



?>
