<?php

/* urls */
define ('pw_was_js_url', pw_was_url . '/js');


require_once('Defwa.php');
require_once('Custwa.php');
require_once('CustwaList.php');
require_once('DefwaList.php');
require_once('DefwaList.php');
require_once('WasAjaxHandler.php');
require_once(pw_was . '/gui/WasGuiManager.php');
require_once(pw_was . '/WasScriptManager.php');

if(!class_exists('WaManager')){
    
    /**
     * WaManager offers methods to manage widget areas, and specifically to provide the following services:
     *      1) adding custom widget areas
     *      2) remove custom widget areas
     *      3) editing custom widget areas
     *      4) registering custom widget areas
     *
     * @author Jonathan
     */
    class WaManager {
        private $wasGuiManager;    // manages widget areas gui
        private $wasScriptManager;   // was script manager
        private $defwas;
        private $custwas;
        private $wasAjaxHandler;
        

/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * Constructor of WaManager 
         */
        public function __construct() {
            
            /* members */
            $this->wasGuiManager = new WasGuiManager();
            $this->wasScriptManager = new WasScriptManager();
            
            /* actions */
            // register all the new sidebars
            add_action( 'widgets_init', array ($this, 'control_and_perform_custwas_registration'), 11 );
            
            //deal with new custom sidebar request
            //add_action( 'widgets_init', array ($this, 'handle_new_custwa_req'), 11 );
         
            // handle deletion of custwa
            add_action( 'plugins_loaded', array ($this, 'handle_delete_custwa_form_submission') );
            
            $this->wasAjaxHandler = new WasAjaxHandler();
        } // end WaManager Constructor

            
//-------------------------------------------------getters-------------------------------------------------
        
        function get_wasGuiManager(){
            return $this->wasGuiManager;
        }
        
        function get_wasScriptManager(){
            return $this->wasScriptManager;
        }
        
        function get_defwas() {
            return $this->defwas;
        }
        
        function get_custwas(){
            return $this->custwas;
        }
        
        function get_wasAjaxHandler(){
            return $this->wasAjaxHandler;
        }
    
//-------------------------------------------------end getters-------------------------------------------------    
//-------------------------------------------------setters-------------------------------------------------  

        function set_wasGuiManager($value) {
            $this->wasGuiManager = $value;
        }
        
        function set_wasScriptManager($value) {
            $this->wasScriptManager = $value;
        }
        
        function set_defwas($value) {
            $this->defwas = $value;
        }
        
        function set_custwas($value) {
            $this->custwas = $value;
        }
        
        function set_wasAjaxHandler($value) {
            $this->wasAjaxHandler = $value;
        }
        
    
//-------------------------------------------------end setters-------------------------------------
            
            
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
            /**
             * adds a custwa widget area
             * @param string $cs_name
             *      custom widget area name
             * @param string $cs_decript
             *      custom widget area description
             * @param string $cs_theme
             *      custom widget area theme
             * @param string $cs_class
             *      custom widget area class
             * @param string $cs_bewidget
             *      custom widget area before widget
             * @param string $cs_afwidget
             *      custom widget area after widget
             * @param string $cs_betitle
             *      custom widget area before title
             * @param string $cs_aftitle
             *      custom widget area after title
             * 
             * @return boolean is custom widget area was successfully added
             *      true - if custwa was added successfully.
             *      false - otherwise
             */
            public function je_sb_add_custwa($cs_name, $cs_decript, $cs_theme, $cs_class = '', $cs_bewidget = '', $cs_afwidget = '', $cs_betitle = '', $cs_aftitle = '') {
                $new_custwa = null;
                
                // if custwa is valid
                if ($this->valid_custwa($cs_name)){
                    $new_custwa = new Custwa($cs_name, $cs_name, $cs_decript, $cs_theme, $cs_class, $cs_bewidget, $cs_afwidget, $cs_betitle, $cs_aftitle);
                }
                
                // if custwa was created successfully
                if ($new_custwa) {
                    // add new custwa to the db
                    DbFunctionsLite::insert_new_custwa($new_custwa);
                    return TRUE;
                }
                // else return false
                else{
                    return FALSE;
                }
            } // end function je_sb_add_custwa

/* ---------------------------------------------------------------------------------------------------------------------------------------*/
            /**
             * adds a custwa widget area
             * @param string $cs_name
             *      custom widget area name
             * @param string $cs_decript
             *      custom widget area description
             * @param string $cs_theme
             *      custom widget area theme
             * @param string $cs_class
             *      custom widget area class
             * @param string $cs_bewidget
             *      custom widget area before widget
             * @param string $cs_afwidget
             *      custom widget area after widget
             * @param string $cs_betitle
             *      custom widget area before title
             * @param string $cs_aftitle
             *      custom widget area after title
             * 
             * @return boolean is custom widget area was successfully added
             *      true - if custwa was added successfully.
             *      false - otherwise
             */
            public function editCustwa($oldCustwaName, $newCustwa)  {
                
                // if custwa name is valid (or it has not changed
                if ($this->valid_custwa($newCustwa->get_name()) || $oldCustwaName == $newCustwa->get_name()){
                    
                    // change custwa record in custwas table
                    DbFunctionsLite::updateCustwaInCustwasTable($oldCustwaName, $newCustwa);
                    
                    // change custwa name for all custwa repwas
                    DbFunctionsLite::updateCustwaInRepwasTable($oldCustwaName, $newCustwa);
                    
                    return TRUE;
                }
                 return FALSE;

            } // end function je_sb_add_custwa
            
            
 /*--------------------------------------------------Custwa Validation Functions------------------------------------------------*/
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * checks if custwa is valid
         * @param string $cs_name
         *      custwa name
         * @param string $description
         *      custwa description
         * @return boolean custwa name validity
         * TRUE - if custwa name is valid
         * FALSE - otherwise
         */
        function valid_custwa($cs_name) {
            // check custwa name validity
            $valid_name = $this->check_custwa_name_validity($cs_name);

            // check entered widget area is unique
            $valid_uniqueness = $this->check_entered_wa_unique($cs_name);

            return $valid_name && $valid_uniqueness;

            return false;
        } // end function valid_custwa

/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * checks if custwa name is valid. Specifically, name is composed only of: letters, numbers and underscores
         * @param string $custwa_name
         *      custwa name
         * @return boolean custwa name validity
         *      TRUE - if custwa name is valid
         *      FALSE - otherwise
         */
        function check_custwa_name_validity ($custwa_name) {
                $legal_regex_pattern = '#^[a-z0-9]+[a-z0-9- ]*$#i';

                if (!preg_match($legal_regex_pattern, $custwa_name)){
                    NotificationManager::je_sb_add_error(new WP_Error('custwa_name_invalid', __("Custom Widget Area name is invalid. Valid name should start with a letter or number, followed by letters, numbers, spaces or hyphens.")));
                    return FALSE;
                }
                return TRUE;
        } // end function check_custwa_name_validity

/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
            * checks if $cs_name is unique, i.e. does not exist in the db both as defwa and as custwa
            * @param string $cs_name 
            *       custwa name
            * @return boolean
            *      custwa uniqueness (concerning both defwas and custwas)
            *      Return Values:
            *           TRUE - if custwa name is unique (concerning both defwas and custwas)
            *           FALSE - otherwise
         */
        function check_entered_wa_unique($cs_name) {
                //get all custwas
                $all_custwas = DbFunctionsLite::get_custwas();
                // check if there's a custwa with cs_name == $cs_name
                $is_custwa_exists = $this->is_custwa_exists_in_custwas($all_custwas, $cs_name);

                // get all defwas (all because we don't want a custwa name to be any possible defwa name)
                $all_defwas = DbFunctionsLite::get_defwas();
                // check if there's a defwa with defwa_id == $cs_name
                $is_defwa_exists = $this->is_custwa_exists_in_defwas($all_defwas, $cs_name);

                return $is_custwa_exists && $is_defwa_exists;
            return false;
        } // end function check_entered_wa_unique

/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * checks if $cs_name is unique as custwa, i.e. does not exist in the db as custwa
         * @param array $all_custwas 
         *      array containing all custwas
         * @param string $custwa_name 
         *      custwa name
         * @return boolean 
         *      returns boolean indicating custwa uniqueness as custwa
         *      TRUE - if custwa name is unique as custwa
         *      FALSE - otherwise
         */
        function is_custwa_exists_in_custwas($all_custwas, $custwa_name){
                //for each custwa $curr_custwa that exists in $all_custwas
                foreach ($all_custwas as $curr_custwa) {
                    // if current custwa contains the same name as $cs_name custwa
                    if ($curr_custwa['custwa_name'] === $custwa_name){
                        NotificationManager::je_sb_add_error(new WP_Error('custwa_already_exists', __("Custom Widget Area name already exists. Please enter another name.")));
                        return FALSE;
                    }
                }
                return TRUE;
            return false;
        }  // end function is_custwa_exists_in_custwas


/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * checks if $cs_name is unique as custwa, i.e. does not exist in the db as custwa
         * 
         * @param array $all_defwas
         *      array containing all defwas
         * @param string $defwa_name
         *      defwa name
         * @return boolean custwa uniqueness as defwa
         *      TRUE - if custwa name is unique as defwa
         *      FALSE - otherwise
         */
        function is_custwa_exists_in_defwas($all_defwas, $defwa_name){
                //for each custwa $curr_custwa that exists in $all_custwas
                foreach ($all_defwas as $curr_defwa) {
                    // if current custwa contains the same name as $cs_name custwa
                    if ($curr_defwa['defwa_id'] === $defwa_name){
                        NotificationManager::je_sb_add_error(new WP_Error('custwa_already_exists_as_defwa', __("Custom Widget Area name already exists as Default Widget Area name. Please enter another name.")));
                        return FALSE;
                    }
                }
                return TRUE;
            return FALSE;
        } // end function is_custwa_exists_in_defwas
        
 /*--------------------------------------------------End Of Custwa Validation Functions---------------------------------------*/

/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * deletes custwa widget area
         * @param string $cs_name 
         *      custom widget area name
         * @return boolean
         *       describes if custom widget area was successfully deleted.
         *      Return Values:
         *          true - if custwa was successfully deleted.
         *          false - otherwise
         */
        function je_sb_delete_cs ($cs_name) {
                // delete custwa widgets
                $this->je_sb_delete_sidebar_widgets($cs_name);

                // delete the custwa from replacements_wa table
                DbFunctionsLite::delete_custwa_repwas(wp_get_theme(), $cs_name);

                // delete the custwa from custom_wa table
                DbFunctionsLite::delete_custwa(wp_get_theme(), $cs_name);

                // add good notification: custwa was successfully deleted
                NotificationManager::je_sb_add_good_notification("Custom Widget Area " . $cs_name . " was successfully deleted");

                return true;
        }  // end function je_sb_delete_cs

/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * deletes a given custwa's widgets
         * @param string $sidebar_to_remove 
         *      custom widget area name of which widgets will be removed
         * @return boolean is custom widget area was successfully deleted
         *      true - if custwa's widgets were successfully deleted.
         *      false - otherwise
         */
       function je_sb_delete_sidebar_widgets($sidebar_to_remove){
           // get sidebar widgets option from db
            $sidebar_widgets = get_option('sidebars_widgets', null);
            
            // get array of widgets of deleted custwa
            $deletedCustwaWidgets = $sidebar_widgets[$sidebar_to_remove];
            
            // copy widgets of deleted custwa to inactive widgets
            if ($deletedCustwaWidgets) {
                foreach ($deletedCustwaWidgets as $custwaWidget) {
                    $sidebar_widgets['wp_inactive_widgets'][] = $custwaWidget;
                }
            }
            
            // delete deleted custwa's widgets
            unset($sidebar_widgets[$sidebar_to_remove]);
            
            // update sidebars widgets option
            update_option('sidebars_widgets', $sidebar_widgets);
            return true;
        } // end function je_sb_remove_custwa

/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * registers all custom widget areas as wordpress widget areas
         * @param none
         * @return void
         */
        function control_and_perform_custwas_registration() {
            global $wp_registered_sidebars;
                
                //get custwas from db
                $this->custwas = $this->get_custwa_list_from_db();

                //assign the so far registered default sidebars to defwas member
                $this->defwas = DefwaList::standard_registered_sidebars_to_wa_list($wp_registered_sidebars, wp_get_theme());

                // register custwas
                $this->registerCustwas ($this->custwas);
                
                // now we can be sure that there are registered default sidebars. This is the time to update the defwa table in the db
                DbFunctionsLite::update_defwas_in_db($this->get_defwas());
               
        }   // end function control_and_perform_custwas_registration

        /**
         * registers the cusom widget areas provided in $custwas as wordpress sidebars
         * @param CustwaList $custwas
         */
        function registerCustwas (CustwaList $custwas){
                if ($custwas){
                    foreach ($custwas->getList() as $custwa){
                        register_sidebar(
                                array(
                                    'id'=>$custwa->get_id(), 
                                    'name'=>$custwa->get_name(), 
                                    'description'=>$custwa->get_description(), 
                                    'class'=>$custwa->get_class(), 
                                    'before_widget' => $custwa->get_bewidget(), 
                                    'after_widget' => $custwa->get_afwidget(), 
                                    'before_title' => $custwa->get_betitle(), 
                                    'after_title' => $custwa->get_aftitle()
                                )
                            );
                    }
                }
        }
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * handles new custwa user request.
         * builds new custom sidebar according to the data via form
         * checks if custwa name entered is valid and adds validation message correspondingly.
         * @param none
         * @return void
         */
        function handle_new_custwa_req() {
            if (strcmp($_SERVER['REQUEST_METHOD'], "POST") == 0){
                //if the form that was sent was new cs form - update 
                if (isset($_POST['form_type']) && strcmp($_POST['form_type'], "newCS_form_type") == 0){
                    //get sidebar name
                    $custwaName = $_POST['NewWaNameName'];

                    //get sidebar description
                    $custwaDescription = $_POST['NewWaDescriptionName'];
                    $custwaTheme = wp_get_theme();

                    // try to add custwa
                    $valid_custwa_insert = $GLOBALS['mainPwObj']->get_je_sb_wa_manager()->je_sb_add_custwa($custwaName, $custwaDescription, $custwaTheme);
                    // if successful
                    if ($valid_custwa_insert){
                        // add good notification
                        NotificationManager::je_sb_add_good_notification("New custom widget area was created!");
                    }
                    // else if custwa input in invalid - it is taken care in the input validation function.
                }
            }
        }   //end function handle_new_custwa_req
        
/* -------------------------------------------------------db management-----------------------------------------------------------*/

/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * gets a a list of defwas from db as DefwaList object
         * @param none
         * @return DefwaList list of defwas from db:
         */
        public function get_defwa_list_from_db() {
            
            // initialize custwa list
            $defwa_list = new DefwaList();
            $defwas = DbFunctionsLite::get_defwas();
            //if there are defwas
            if ($defwas){
                foreach ($defwas as $defwa){
                        $defwa_obj = new Defwa (
                            $defwa['defwa_id'], 
                            $defwa['defwa_name'], 
                            $defwa['defwa_description'], 
                            $defwa['defwa_theme'], 
                            $defwa['defwa_class'], 
                            $defwa['defwa_before_widget'], 
                            $defwa['defwa_after_widget'], 
                            $defwa['defwa_before_title'], 
                            $defwa['defwa_after_title']
                    );  // end new defwa
                    $defwa_list->add($defwa_obj);
                }   // end foreach
            }   // end if defwas exist

            return $defwa_list;
        }   // end function get_defwa_list_from_db        
        
/* ---------------------------------------------------------------------------------------------------------------------------------------*/
        /**
         * gets a a list of custwas from db as CustwaList object
         * @param none
         * @return CustwaList list of custwas from db:
         */
        public function get_custwa_list_from_db() {
            // initialize custwa list
            $custwa_list = new CustwaList();
            $custwas = DbFunctionsLite::get_custwas();
            //if there are custom sidebars - register them
            if ($custwas){
                foreach ($custwas as $custwa){
                        $custwa_obj = new Custwa (
                            $custwa['custwa_name'], 
                            $custwa['custwa_name'], 
                            $custwa['custwa_description'], 
                            $custwa['custwa_theme'], 
                            $custwa['custwa_class'], 
                            $custwa['custwa_before_widget'], 
                            $custwa['custwa_after_widget'], 
                            $custwa['custwa_before_title'], 
                            $custwa['custwa_after_title']
                    );  // end new Custwa
                    $custwa_list->add($custwa_obj);
                }   // end foreach
            }   // end if custwas exist

            return $custwa_list;
        }   // end function get_custwa_list_from_db
/* -------------------------------------------------------end db management-----------------------------------------------------------*/
        
        /**
         * handles a delete custwa form submission
         * @param none
         * @return void
         */
        function handle_delete_custwa_form_submission(){
            //if a delete form has been sent
            if ( $_SERVER['REQUEST_METHOD'] === "POST" &&
                        isset($_POST['form_type']) &&
                        $_POST['form_type'] === "CSDelete_form_type"
                )
            {
                    $cs_id = $_POST['cs_select_box'];
                    $this->je_sb_delete_cs($cs_id );
            }   // end if delete form has been sent
        } // end function handle_delete_custwa_form_submission


        
    } // end class WaManager
}   // end if class exists

?>
